@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: eval.c,v 1.96.2.3 2012/09/14 21:13:08 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: eval.c,v 1.119.2.4 2016/08/18 17:23:10 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - eval.c */
@@ -43,11 +43,15 @@ static char *RCSid() { return RCSid("$Id: eval.c,v 1.96.2.3 2012/09/14 21:13:08
 #include "syscfg.h"
 #include "alloc.h"
 #include "datafile.h"
+#include "datablock.h"
+#include "external.h"	/* for f_calle */
 #include "internal.h"
+#include "libcerf.h"
 #include "specfun.h"
 #include "standard.h"
 #include "util.h"
 #include "version.h"
+#include "term_api.h"
 
 #include <signal.h>
 #include <setjmp.h>
@@ -61,6 +65,8 @@ struct udvt_entry *udv_NaN;
 /* first in linked list */
 struct udvt_entry *first_udv = &udv_pi;
 struct udft_entry *first_udf = NULL;
+/* pointer to first udv users can delete */
+struct udvt_entry **udv_user_head;
 
 TBOOLEAN undefined;
 
@@ -72,12 +78,7 @@ static int s_p = -1;		/* stack pointer */
 static int jump_offset;		/* to be modified by 'jump' operators */
 
 /* The table of built-in functions */
-/* HBB 20010725: I've removed all the casts to type (FUNC_PTR) ---
- * According to ANSI/ISO C Standards it causes undefined behaviour if
- * you cast a function pointer to any other type, including a function
- * pointer with a different set of arguments, and then call the
- * function.  Instead, I made all these functions adhere to the common
- * type, directly */
+/* These must strictly parallel enum operators in eval.h */
 const struct ft_entry GPFAR ft[] =
 {
     /* internal functions: */
@@ -104,6 +105,8 @@ const struct ft_entry GPFAR ft[] =
     {"lt",  f_lt},
     {"ge",  f_ge},
     {"le",  f_le},
+    {"leftshift", f_leftshift},
+    {"rightshift", f_rightshift},
     {"plus",  f_plus},
     {"minus",  f_minus},
     {"mult",  f_mult},
@@ -123,9 +126,13 @@ const struct ft_entry GPFAR ft[] =
     {"jumpnz",  f_jumpnz},
     {"jtern",  f_jtern},
 
-/* Placeholder for FS_START */
+/* Placeholder for SF_START */
     {"", NULL},
 
+#ifdef HAVE_EXTERNAL_FUNCTIONS
+    {"", f_calle},
+#endif
+
 /* legal in using spec only */
     {"column",  f_column},
     {"stringcolumn",  f_stringcolumn},	/* for using specs */
@@ -173,9 +180,7 @@ const struct ft_entry GPFAR ft[] =
     {"rand",  f_rand},
     {"floor",  f_floor},
     {"ceil",  f_ceil},
-#ifdef BACKWARDS_COMPATIBLE
-    {"defined",  f_exists},       /* deprecated syntax defined(foo) */
-#endif
+    {"defined", f_exists},	/* DEPRECATED syntax defined(foo) */
 
     {"norm",  f_normal},	/* XXX-JG */
     {"inverf",  f_inverse_erf},	/* XXX-JG */
@@ -187,6 +192,14 @@ const struct ft_entry GPFAR ft[] =
     {"airy",  f_airy},         /* janert, 20090905 */
     {"expint",  f_expint},     /* Jim Van Zandt, 20101010 */
 
+#ifdef HAVE_LIBCERF
+    {"cerf", f_cerf},		/* complex error function */
+    {"cdawson", f_cdawson},	/* complex Dawson's integral */
+    {"erfi", f_erfi},		/* imaginary error function */
+    {"VP", f_voigtp},		/* Voigt profile */
+    {"faddeeva", f_faddeeva},	/* Faddeeva rescaled complex error function "w_of_z" */
+#endif
+
     {"tm_sec",  f_tmsec},	/* for timeseries */
     {"tm_min",  f_tmmin},	/* for timeseries */
     {"tm_hour",  f_tmhour},	/* for timeseries */
@@ -201,7 +214,7 @@ const struct ft_entry GPFAR ft[] =
     {"strlen",  f_strlen},	/* for string variables only */
     {"strstrt",  f_strstrt},	/* for string variables only */
     {"substr",  f_range},	/* for string variables only */
-    {"word", f_words},		/* for string variables only */
+    {"word",  f_word},		/* for string variables only */
     {"words", f_words},		/* implemented as word(s,-1) */
     {"strftime",  f_strftime},  /* time to string */
     {"strptime",  f_strptime},  /* string to time */
@@ -211,6 +224,8 @@ const struct ft_entry GPFAR ft[] =
     {"exists", f_exists},	/* exists("foo") replaces defined(foo) */
     {"value", f_value},		/* retrieve value of variable known by name */
 
+    {"hsv2rgb", f_hsv2rgb},	/* color conversion */
+
     {NULL, NULL}
 };
 
@@ -259,6 +274,25 @@ real(struct value *val)
 }
 
 
+/* returns the real part of val, converted to int if necessary */
+int
+real_int(struct value *val)
+{
+    switch (val->type) {
+    case INTGR:
+	return val->v.int_val;
+    case CMPLX:
+	return (int) val->v.cmplx_val.real;
+    case STRING:
+	return atoi(val->v.string_val);
+    default:
+	int_error(NO_CARET, "unknown type in real_int()");
+    }
+    /* NOTREACHED */
+    return 0;
+}
+
+
 /* returns the imag part of val */
 double
 imag(struct value *val)
@@ -448,16 +482,29 @@ struct value *
 pop_or_convert_from_string(struct value *v)
 {
     (void) pop(v);
+
+    /* DEBUG Dec 2014 - Consolidate sanity check for variable type */
+    /* FIXME: Test for INVALID_VALUE? Other corner cases? */
+    if (v->type == INVALID_NAME)
+	int_error(NO_CARET, "invalid dummy variable name");
+
     if (v->type == STRING) {
 	char *eov;
-	double d = strtod(v->v.string_val,&eov);
-	if (v->v.string_val == eov) {
+
+	if (strspn(v->v.string_val,"0123456789 ") == strlen(v->v.string_val)) {
+	    int i = atoi(v->v.string_val);
 	    gpfree_string(v);
-	    int_error(NO_CARET,"Non-numeric string found where a numeric expression was expected");
+	    Ginteger(v, i);
+	} else {
+	    double d = strtod(v->v.string_val,&eov);
+	    if (v->v.string_val == eov) {
+		gpfree_string(v);
+		int_error(NO_CARET,"Non-numeric string found where a numeric expression was expected");
+	    }
+	    gpfree_string(v);
+	    Gcomplex(v, d, 0.);
+	    FPRINTF((stderr,"converted string to CMPLX value %g\n",real(v)));
 	}
-	gpfree_string(v);
-	Gcomplex(v, d, 0.);
-	FPRINTF((stderr,"converted string to CMPLX value %g\n",real(v)));
     }
     return(v);
 }
@@ -580,12 +627,12 @@ execute_at(struct at_type *at_ptr)
     jump_offset = saved_jump_offset;
 }
 
-/* 20010724: moved here from parse.c, where it didn't belong */
+/* May 2013: Old hackery #ifdef'ed out so that input of Inf/NaN */
+/* values through evaluation is treated equivalently to direct  */
+/* input of a formated value.  See revised imageNaN demo.       */
 void
 evaluate_at(struct at_type *at_ptr, struct value *val_ptr)
 {
-    double temp = 0;
-
     undefined = FALSE;
     errno = 0;
     reset_stack();
@@ -598,20 +645,26 @@ evaluate_at(struct at_type *at_ptr, struct value *val_ptr)
 
     execute_at(at_ptr);
 
-    if (!evaluate_inside_using || !df_nofpe_trap) {
+    if (!evaluate_inside_using || !df_nofpe_trap)
 	(void) signal(SIGFPE, SIG_DFL);
+
+    if (errno == EDOM || errno == ERANGE)
+	undefined = TRUE;
+#if (1)	/* New code */
+    else if (!undefined) {
+	(void) pop(val_ptr);
+	check_stack();
     }
 
-    if (errno == EDOM || errno == ERANGE) {
-	undefined = TRUE;
-    } else if (!undefined) {	/* undefined (but not errno) may have been set by matherr */
+#else /* Old hackery */
+    else if (!undefined) { /* undefined (but not errno) may have been set by matherr */
 	(void) pop(val_ptr);
 	check_stack();
 	/* At least one machine (ATT 3b1) computes Inf without a SIGFPE */
-	if (val_ptr->type != STRING)
-	temp = real(val_ptr);
-	if (temp > VERYLARGE || temp < -VERYLARGE) {
-	    undefined = TRUE;
+	if (val_ptr->type != STRING) {
+	    double temp = real(val_ptr);
+	    if (temp > VERYLARGE || temp < -VERYLARGE)
+		undefined = TRUE;
 	}
     }
 #if defined(NeXT) || defined(ultrix)
@@ -626,11 +679,11 @@ evaluate_at(struct at_type *at_ptr, struct value *val_ptr)
 	Gcomplex(val_ptr, 0.0 / 0.0, 0.0 / 0.0);
     }
 #endif /* NeXT || ultrix */
-
+#endif /* old hackery */
 }
 
 void
-free_at(struct at_type *at_ptr)
+real_free_at(struct at_type *at_ptr)
 {
     int i;
     /* All string constants belonging to this action table have to be
@@ -644,9 +697,14 @@ free_at(struct at_type *at_ptr)
 	    gpfree_string(&(a->arg.v_arg));
 	/* a summation contains its own action table wrapped in a private udf */
 	if (a->index == SUM) {
-	    free_at(a->arg.udf_arg->at);
+	    real_free_at(a->arg.udf_arg->at);
 	    free(a->arg.udf_arg);
 	}
+#ifdef HAVE_EXTERNAL_FUNCTIONS
+	/* external function calls contain a parameter list */
+	if (a->index == CALLE)
+	    free(a->arg.exf_arg);
+#endif
     }
     free(at_ptr);
 }
@@ -691,23 +749,32 @@ get_udv_by_name(char *key)
     return NULL;
 }
 
+/* This doesn't really delete, it just marks the udv as undefined */
 void
-del_udv_by_name( char *key, TBOOLEAN wildcard )
+del_udv_by_name(char *key, TBOOLEAN wildcard)
 {
-    struct udvt_entry *udv_ptr = first_udv;
+    struct udvt_entry *udv_ptr = *udv_user_head;
 
     while (udv_ptr) {
+	/* Forbidden to delete GPVAL_* */
+	if (!strncmp(udv_ptr->udv_name,"GPVAL",5))
+	    ;
+	else if (!strncmp(udv_ptr->udv_name,"GNUTERM",7))
+	    ;
+
  	/* exact match */
-	if (!wildcard && !strcmp(key, udv_ptr->udv_name)) {
+	else if (!wildcard && !strcmp(key, udv_ptr->udv_name)) {
 	    udv_ptr->udv_undef = TRUE;
 	    gpfree_string(&(udv_ptr->udv_value));
+	    gpfree_datablock(&(udv_ptr->udv_value));
 	    break;
 	}
 
 	/* wildcard match: prefix matches */
-	if ( wildcard && !strncmp(key, udv_ptr->udv_name, strlen(key)) ) {
+	else if ( wildcard && !strncmp(key, udv_ptr->udv_name, strlen(key)) ) {
 	    udv_ptr->udv_undef = TRUE;
 	    gpfree_string(&(udv_ptr->udv_value));
+	    gpfree_datablock(&(udv_ptr->udv_value));
 	    /* no break - keep looking! */
 	}
 
@@ -715,6 +782,24 @@ del_udv_by_name( char *key, TBOOLEAN wildcard )
     }
 }
 
+/* Clear (delete) all user defined functions */
+void
+clear_udf_list()
+{
+    struct udft_entry *udf_ptr = first_udf;
+    struct udft_entry *udf_next;
+
+    while (udf_ptr) {
+	free(udf_ptr->udf_name);
+	free(udf_ptr->definition);
+	free_at(udf_ptr->at);
+	udf_next = udf_ptr->next_udf;
+	free(udf_ptr);
+	udf_ptr = udf_next;
+    }
+    first_udf = NULL;
+}
+
 static void update_plot_bounds __PROTO((void));
 static void fill_gpval_axis __PROTO((AXIS_INDEX axis));
 static void set_gpval_axis_sth_double __PROTO((const char *prefix, AXIS_INDEX axis, const char *suffix, double value, int is_int));
@@ -724,10 +809,12 @@ set_gpval_axis_sth_double(const char *prefix, AXIS_INDEX axis, const char *suffi
 {
     struct udvt_entry *v;
     char *cc, s[24];
-    sprintf(s, "%s_%s_%s", prefix, axis_defaults[axis].name, suffix);
-    for (cc=s; *cc; cc++) *cc = toupper(*cc); /* make the name uppercase */
+    sprintf(s, "%s_%s_%s", prefix, axis_name(axis), suffix);
+    for (cc=s; *cc; cc++)
+	*cc = toupper((unsigned char)*cc); /* make the name uppercase */
     v = add_udv_by_name(s);
-    if (!v) return; /* should not happen */
+    if (!v) 
+	return; /* should not happen */
     v->udv_undef = FALSE;
     if (is_int)
 	Ginteger(&v->udv_value, (int)(value+0.5));
@@ -742,11 +829,6 @@ fill_gpval_axis(AXIS_INDEX axis)
     AXIS *ap = &axis_array[axis];
     double a = AXIS_DE_LOG_VALUE(axis, ap->min);
     double b = AXIS_DE_LOG_VALUE(axis, ap->max);
-    if ((ap->range_flags & RANGE_REVERSE) && (a > b)) {
-	double tmp = a;
-	a = b;
-	b = tmp;
-    }
     set_gpval_axis_sth_double(prefix, axis, "MIN", a, 0);
     set_gpval_axis_sth_double(prefix, axis, "MAX", b, 0);
     set_gpval_axis_sth_double(prefix, axis, "LOG", ap->base, 0);
@@ -818,6 +900,7 @@ update_plot_bounds(void)
     fill_gpval_integer("GPVAL_TERM_YMAX", axis_array[FIRST_Y_AXIS].term_upper / term->tscale);
     fill_gpval_integer("GPVAL_TERM_XSIZE", canvas.xright+1);
     fill_gpval_integer("GPVAL_TERM_YSIZE", canvas.ytop+1);
+    fill_gpval_integer("GPVAL_TERM_SCALE", term->tscale);
 }
 
 /*
@@ -846,6 +929,7 @@ update_gpval_variables(int context)
 	fill_gpval_axis(U_AXIS);
 	fill_gpval_axis(V_AXIS);
 	fill_gpval_float("GPVAL_R_MIN", R_AXIS.min);
+	fill_gpval_float("GPVAL_R_MAX", R_AXIS.max);
 	fill_gpval_float("GPVAL_R_LOG", R_AXIS.base);
 	update_plot_bounds();
 	fill_gpval_integer("GPVAL_PLOT", is_3d_plot ? 0:1);
@@ -871,6 +955,7 @@ update_gpval_variables(int context)
 	fill_gpval_string("GPVAL_TERMOPTIONS", term_options);
 	fill_gpval_string("GPVAL_OUTPUT", (outstr) ? outstr : "");
 	fill_gpval_string("GPVAL_ENCODING", encoding_names[encoding]);
+	fill_gpval_string("GPVAL_MINUS_SIGN", minus_sign ? minus_sign : "-");
     }
 
     /* If we are called from int_error() then set the error state */
@@ -934,10 +1019,9 @@ gp_words(char *string)
 {
     struct value a;
 
-	push(Gstring(&a,string));
-	push(Ginteger(&a,-1));
-	f_words((union argument *)NULL);
-	pop(&a);
+    push(Gstring(&a, string));
+    f_words((union argument *)NULL);
+    pop(&a);
 
     return a.v.int_val;
 }
@@ -948,11 +1032,34 @@ gp_word(char *string, int i)
 {
     struct value a;
 
-	push(Gstring(&a,string));
-	push(Ginteger(&a,i));
-	f_words((union argument *)NULL);
-	pop(&a);
-	
+    push(Gstring(&a, string));
+    push(Ginteger(&a, i));
+    f_word((union argument *)NULL);
+    pop(&a);
+
     return a.v.string_val;
 }
 
+
+/* Evaluate the function linking secondary axis to primary axis */
+double
+eval_link_function(int axis, double raw_coord)
+{
+    udft_entry *link_udf = axis_array[axis].link_udf;
+    int dummy_var;
+    struct value a;
+
+    if (axis == FIRST_Y_AXIS || axis == SECOND_Y_AXIS)
+	dummy_var = 1;
+    else
+	dummy_var = 0;
+    link_udf->dummy_values[1-dummy_var].type = INVALID_NAME;
+
+    Gcomplex(&link_udf->dummy_values[dummy_var], raw_coord, 0.0);
+    evaluate_at(link_udf->at, &a);
+
+    if (a.type != CMPLX)
+	a = udv_NaN->udv_value;
+
+    return a.v.cmplx_val.real;
+}
