@@ -52,6 +52,7 @@
 /* Default point size is taken from the global "pointsize" variable */
 #define PTSZ_DEFAULT    (-2)
 #define PTSZ_VARIABLE   (-3)
+#define AS_VARIABLE	(-3)
 
 /* Type definitions */
 
@@ -78,31 +79,46 @@ typedef struct text_label {
     struct text_label *next;	/* pointer to next label in linked list */
     int tag;			/* identifies the label */
     t_position place;
-    enum JUSTIFY pos;
+    enum JUSTIFY pos;		/* left/center/right horizontal justification */
     int rotate;
     int layer;
+    int boxed;			/* EAM_BOXED_TEXT */
     char *text;
     char *font;			/* Entry font added by DJL */
     struct t_colorspec textcolor;
     struct lp_style_type lp_properties;
     struct position offset;
     TBOOLEAN noenhanced;
+    TBOOLEAN hypertext;
 } text_label;
 
 /* This is the default state for the axis, timestamp, and plot title labels
  * indicated by tag = -2 */
+#define NONROTATABLE_LABEL_TAG -2
+#define ROTATE_IN_3D_LABEL_TAG -3
+#define VARIABLE_ROTATE_LABEL_TAG -4
 #define EMPTY_LABELSTRUCT \
-    {NULL, -2, {character, character, character, 0.0, 0.0, 0.0}, CENTRE, 0, 0, \
+    {NULL, NONROTATABLE_LABEL_TAG, \
+     {character, character, character, 0.0, 0.0, 0.0}, CENTRE, 0, 0, \
+     0, \
      NULL, NULL, {TC_LT, -2, 0.0}, DEFAULT_LP_STYLE_TYPE, \
-     {character, character, character, 0.0, 0.0, 0.0}, FALSE }
+     {character, character, character, 0.0, 0.0, 0.0}, FALSE, \
+     FALSE}
 
 /* Datastructure for implementing 'set arrow' */
+typedef enum arrow_type {
+    arrow_end_absolute,
+    arrow_end_relative,
+    arrow_end_oriented
+    } arrow_type;
+
 typedef struct arrow_def {
     struct arrow_def *next;	/* pointer to next arrow in linked list */
     int tag;			/* identifies the arrow */
+    arrow_type type;		/* how to interpret t_position end */
     t_position start;
     t_position end;
-    TBOOLEAN relative;		/* second coordinate is relative to first */
+    double angle;		/* angle in degrees if type arrow_end_oriented */
     struct arrow_style_type arrow_properties;
 } arrow_def;
 
@@ -116,16 +132,22 @@ typedef struct rectangle {
     t_position tr;		/* top right */
 } t_rectangle;
 
+#define DEFAULT_RADIUS (-1.0)
+#define DEFAULT_ELLIPSE (-2.0)
 typedef struct circle {
     int type;			/* not used */
     t_position center;		/* center */
     t_position extent;		/* radius */
     double arc_begin;
     double arc_end;
+    TBOOLEAN wedge;		/* TRUE = connect arc ends to center */
 } t_circle;
 
+#define ELLIPSEAXES_XY (0)
+#define ELLIPSEAXES_XX (1)
+#define ELLIPSEAXES_YY (2)
 typedef struct ellipse {
-    int type;			/* not used */
+    int type;			/* mapping of axes: ELLIPSEAXES_XY, ELLIPSEAXES_XX or ELLIPSEAXES_YY */
     t_position center;		/* center */
     t_position extent;		/* major and minor axes */
     double orientation;		/* angle of first axis to horizontal */
@@ -136,12 +158,19 @@ typedef struct polygon {
     t_position *vertex;		/* Array of vertices */
 } t_polygon;
 
+typedef enum en_clip_object {
+    OBJ_CLIP,		/* Clip to graph unless coordinate type is screen */
+    OBJ_NOCLIP,	/* Clip to canvas, never to graph */
+    OBJ_ALWAYS_CLIP	/* Not yet implemented */
+} t_clip_object;
+
 /* Datastructure for 'set object' */
 typedef struct object {
     struct object *next;
     int tag;
     int layer;			/* behind or back or front */
     int object_type;		/* OBJ_RECTANGLE */
+    t_clip_object clip;         
     fill_style_type fillstyle;
     lp_style_type lp_properties;
     union o {t_rectangle rectangle; t_circle circle; t_ellipse ellipse; t_polygon polygon;} o;
@@ -152,6 +181,14 @@ typedef struct object {
 #define OBJ_POLYGON (4)
 #endif
 
+/* Datastructure implementing 'set dashtype' */
+struct custom_dashtype_def {
+    struct custom_dashtype_def *next;	/* pointer to next dashtype in linked list */
+    int tag;			/* identifies the dashtype */
+    int d_type;                 /* for DASHTYPE_SOLID or CUSTOM */
+    struct t_dashtype dashtype;
+};
+
 /* Datastructure implementing 'set style line' */
 struct linestyle_def {
     struct linestyle_def *next;	/* pointer to next linestyle in linked list */
@@ -166,6 +203,14 @@ struct arrowstyle_def {
     struct arrow_style_type arrow_properties;
 };
 
+/* For 'set style parallelaxis' */
+struct pa_style {
+    lp_style_type lp_properties;/* used to draw the axes themselves */
+    int layer;			/* front/back */
+};
+#define DEFAULT_PARALLEL_AXIS_STYLE \
+	{{0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 2.0, 0.0, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}, LAYER_FRONT }
+
 /* The stacking direction of the key box: (vertical, horizontal) */
 typedef enum en_key_stack_direction {
     GPKEY_VERTICAL,
@@ -226,8 +271,39 @@ typedef enum histogram_type {
 	HT_CLUSTERED,
 	HT_ERRORBARS
 } t_histogram_type;
-#define DEFAULT_HISTOGRAM_STYLE { HT_NONE, 2, 1, 0.0, 0.0, LT_UNDEFINED, LT_UNDEFINED, 0, NULL, EMPTY_LABELSTRUCT }
+#define DEFAULT_HISTOGRAM_STYLE { HT_CLUSTERED, 2, 1, 0.0, 0.0, LT_UNDEFINED, LT_UNDEFINED, 0, NULL, EMPTY_LABELSTRUCT }
 
+typedef enum en_boxplot_factor_labels {
+	BOXPLOT_FACTOR_LABELS_OFF,
+	BOXPLOT_FACTOR_LABELS_AUTO,
+	BOXPLOT_FACTOR_LABELS_X,
+	BOXPLOT_FACTOR_LABELS_X2
+} t_boxplot_factor_labels;
+
+#define DEFAULT_BOXPLOT_FACTOR -1
+
+typedef struct boxplot_style {
+    int limit_type;	/* 0 = multiple of interquartile 1 = fraction of points */
+    double limit_value;
+    TBOOLEAN outliers;
+    int pointtype;
+    int plotstyle;	/* CANDLESTICKS or FINANCEBARS */
+    double separation;	/* of boxplots if there are more than one factors */
+    t_boxplot_factor_labels labels;	/* Which axis to put the tic labels if there are factors */
+    TBOOLEAN sort_factors;	/* Sort factors in alphabetical order? */
+} boxplot_style;
+extern boxplot_style boxplot_opts;
+#define DEFAULT_BOXPLOT_STYLE { 0, 1.5, TRUE, 6, CANDLESTICKS, 1.0, BOXPLOT_FACTOR_LABELS_AUTO, FALSE }
+
+#ifdef EAM_BOXED_TEXT
+typedef struct textbox_style {
+    TBOOLEAN opaque;	/* True if the box is background-filled before writing into it */
+    TBOOLEAN noborder;	/* True if you want fill only, no lines */
+    double xmargin;	/* fraction of default margin to use */
+    double ymargin;	/* fraction of default margin to use */
+} textbox_style;
+#define DEFAULT_TEXTBOX_STYLE { FALSE, FALSE, 1.0, 1.0 }
+#endif
 
 /***********************************************************/
 /* Variables defined by gadgets.c needed by other modules. */
@@ -262,21 +338,22 @@ typedef struct {
     double width_fix;		/* user specified additional (+/-) width of key titles */
     double height_fix;
     keytitle_type auto_titles;	/* auto title curves unless plotted 'with notitle' */
+    TBOOLEAN front;		/* draw key in a second pass after the rest of the graph */
     TBOOLEAN reverse;		/* key back to front */
     TBOOLEAN invert;		/* key top to bottom */
     TBOOLEAN enhanced;		/* enable/disable enhanced text of key titles */
     struct lp_style_type box;	/* linetype of box around key:  */
-    char title[MAX_LINE_LEN+1];	/* title line for the key as a whole */
     char *font;			/* Will be used for both key title and plot titles */
     struct t_colorspec textcolor;	/* Will be used for both key title and plot titles */
     BoundingBox bounds;
     int maxcols;		/* maximum no of columns for horizontal keys */
     int maxrows;		/* maximum no of rows for vertical keys */
+    text_label title;		/* holds title line for the key as a whole */
 } legend_key;
 
 extern legend_key keyT;
 
-# define DEFAULT_KEYBOX_LP { 0, LT_NODRAW, 0, 1.0, 1.0, 0 }
+#define DEFAULT_KEYBOX_LP {0, LT_NODRAW, 0, DASHTYPE_SOLID, 0, 1.0, PTSZ_DEFAULT, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}
 
 #define DEFAULT_KEY_POSITION { graph, graph, graph, 0.9, 0.9, 0. }
 
@@ -288,10 +365,11 @@ extern legend_key keyT;
 		GPKEY_RIGHT, GPKEY_VERTICAL, \
 		4.0, 1.0, 0.0, 0.0, \
 		FILENAME_KEYTITLES, \
-		FALSE, FALSE, TRUE, \
+		FALSE, FALSE, FALSE, TRUE, \
 		DEFAULT_KEYBOX_LP, \
-		"", \
-		NULL, {TC_LT, LT_BLACK, 0.0} }
+		NULL, {TC_LT, LT_BLACK, 0.0}, \
+		{0,0,0,0}, 0, 0, \
+		EMPTY_LABELSTRUCT}
 
 
 /*
@@ -317,12 +395,20 @@ typedef struct {
   int xoffset;	/* To adjust left or right, e.g. for y2tics */
   struct position origin;
   struct position size;
+  TBOOLEAN invert;	/* gradient low->high runs top->bot rather than bot->top */
   BoundingBox bounds;
 } color_box_struct;
 
 extern color_box_struct color_box;
 extern color_box_struct default_color_box;
 
+/* Holder for various image properties */
+typedef struct t_image {
+    t_imagecolor type; /* See above */
+    TBOOLEAN fallback; /* true == don't use terminal-specific code */
+    unsigned int ncols, nrows; /* image dimensions */
+} t_image;
+
 extern BoundingBox plot_bounds;	/* Plot Boundary */
 extern BoundingBox canvas; 	/* Writable area on terminal */
 extern BoundingBox *clip_area;	/* Current clipping box */
@@ -339,17 +425,20 @@ extern int aspect_ratio_3D;	/* 2 for equal scaling of x and y; 3 for z also */
 extern t_position lmargin, bmargin, rmargin, tmargin;
 #define DEFAULT_MARGIN_POSITION {character, character, character, -1, -1, -1}
 
-extern FILE *table_outfile;
-extern TBOOLEAN table_mode;
+extern struct custom_dashtype_def *first_custom_dashtype;
 
 extern struct arrow_def *first_arrow;
 
 extern struct text_label *first_label;
 
 extern struct linestyle_def *first_linestyle;
+extern struct linestyle_def *first_perm_linestyle;
+extern struct linestyle_def *first_mono_linestyle;
 
 extern struct arrowstyle_def *first_arrowstyle;
 
+extern struct pa_style parallel_axis_style;
+
 #ifdef EAM_OBJECTS
 extern struct object *first_object;
 #endif
@@ -386,6 +475,7 @@ extern int user_border;
 extern int border_layer;
 
 extern struct lp_style_type border_lp;
+extern const struct lp_style_type background_lp;
 extern const struct lp_style_type default_border_lp;
 
 extern TBOOLEAN	clip_lines1;
@@ -410,16 +500,23 @@ extern TBOOLEAN is_3d_plot;
    either the plot is a 2D plot, or it is a suitably oriented 3D plot (e.g. map).
 */
 #define ALMOST2D      \
-    ( !is_3d_plot ||  \
+    ( !is_3d_plot || splot_map || \
       ( fabs(fmod(surface_rot_z,90.0))<0.1  \
         && fabs(fmod(surface_rot_x,180.0))<0.1 ) )
 
-#ifdef VOLATILE_REFRESH
-extern int refresh_ok;		/* 0 = no;  2 = 2D ok;  3 = 3D ok */
+typedef enum E_Refresh_Allowed {
+   E_REFRESH_NOT_OK = 0,
+   E_REFRESH_OK_2D = 2,
+   E_REFRESH_OK_3D = 3
+} TRefresh_Allowed;
+
+extern TRefresh_Allowed refresh_ok;
+# define SET_REFRESH_OK(ok, nplots) do { \
+   refresh_ok = (ok); \
+   refresh_nplots = (nplots); \
+} while(0)
 extern int refresh_nplots;
-#else
-#define refresh_ok FALSE
-#endif
+
 extern TBOOLEAN volatile_data;
 
 /* WINDOWID to be filled by terminals running on X11 (x11, wxt, qt, ...) */
@@ -430,14 +527,16 @@ extern int current_x11_windowid;
 #define LAYER_BEHIND     -1
 #define LAYER_BACK        0
 #define LAYER_FRONT       1
+#define LAYER_FOREGROUND  2	/* not currently used */
 #define LAYER_PLOTLABELS 99
 
 /* Functions exported by gadgets.c */
 
 /* moved here from util3d: */
 void draw_clip_line __PROTO((int, int, int, int));
+void draw_clip_polygon __PROTO((int , gpiPoint *));
 void draw_clip_arrow __PROTO((int, int, int, int, int));
-int clip_line __PROTO((int *, int *, int *, int *));
+void clip_polygon __PROTO((gpiPoint *, gpiPoint *, int , int *));
 int clip_point __PROTO((unsigned int, unsigned int));
 void clip_put_text __PROTO((unsigned int, unsigned int, char *));
 
@@ -454,25 +553,26 @@ extern fill_style_type default_fillstyle;
 #ifdef EAM_OBJECTS
 /*       Warning: C89 does not like the union initializers     */
 extern struct object default_rectangle;
-#define DEFAULT_RECTANGLE_STYLE { NULL, -1, 0, OBJ_RECTANGLE,	\
+#define DEFAULT_RECTANGLE_STYLE { NULL, -1, 0, OBJ_RECTANGLE, OBJ_CLIP,	\
 	{FS_SOLID, 100, 0, BLACK_COLORSPEC},   			\
-	{1, LT_BACKGROUND, 0, 0, 1.0, 0.0, FALSE, DEFAULT_COLORSPEC}, \
+	{0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 1.0, 0.0, 0, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN}, \
 	{.rectangle = {0, {0,0,0,0.,0.,0.}, {0,0,0,0.,0.,0.}, {0,0,0,0.,0.,0.}, {0,0,0,0.,0.,0.}}} }
 
 extern struct object default_circle;
-#define DEFAULT_CIRCLE_STYLE { NULL, -1, 0, OBJ_CIRCLE,       \
+#define DEFAULT_CIRCLE_STYLE { NULL, -1, 0, OBJ_CIRCLE, OBJ_CLIP, \
 	{FS_SOLID, 100, 0, BLACK_COLORSPEC},   			\
-	{1, LT_BACKGROUND, 0, 0, 1.0, 0.0, FALSE, DEFAULT_COLORSPEC},			\
-	{.circle = {1, {0,0,0,0.,0.,0.}, {graph,0,0,0.02,0.,0.}, 0., 360. }} }
+	{0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 1.0, 0.0, 0, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN}, \
+	{.circle = {1, {0,0,0,0.,0.,0.}, {graph,0,0,0.02,0.,0.}, 0., 360., TRUE }} }
 
-#define DEFAULT_ELLIPSE_STYLE { NULL, -1, 0, OBJ_CIRCLE,       \
+extern struct object default_ellipse;
+#define DEFAULT_ELLIPSE_STYLE { NULL, -1, 0, OBJ_ELLIPSE, OBJ_CLIP, \
 	{FS_SOLID, 100, 0, BLACK_COLORSPEC},   			\
-	{1, LT_BACKGROUND, 0, 0, 1.0, 0.0, FALSE, DEFAULT_COLORSPEC}, \
-	{.ellipse = {1, {0,0,0,0.,0.,0.}, {0,0,0,0.,0.,0.}, 0. }} }
+	{0, LT_BACKGROUND, 0, DASHTYPE_SOLID, 0, 1.0, 0.0, 0, BACKGROUND_COLORSPEC, DEFAULT_DASHPATTERN}, \
+	{.ellipse = {ELLIPSEAXES_XY, {0,0,0,0.,0.,0.}, {graph,graph,0,0.05,0.03,0.}, 0. }} }
 
-#define DEFAULT_POLYGON_STYLE { NULL, -1, 0, OBJ_POLYGON,       \
+#define DEFAULT_POLYGON_STYLE { NULL, -1, 0, OBJ_POLYGON, OBJ_CLIP, \
 	{FS_SOLID, 100, 0, BLACK_COLORSPEC},   			\
-	{1, LT_BLACK, 0, 0, 1.0, 0.0, FALSE, DEFAULT_COLORSPEC}, \
+	{0, LT_BLACK, 0, DASHTYPE_SOLID, 0, 1.0, 0.0, 0, BLACK_COLORSPEC, DEFAULT_DASHPATTERN}, \
 	{.polygon = {0, NULL} } }
 
 #endif
@@ -482,16 +582,26 @@ extern filledcurves_opts filledcurves_opts_data;
 extern filledcurves_opts filledcurves_opts_func;
 
 /* Prefer line styles over plain line types */
+#if TRUE || defined(BACKWARDS_COMPATIBLE)
 extern TBOOLEAN prefer_line_styles;
+#else
+#define prefer_line_styles FALSE
+#endif
 
 extern histogram_style histogram_opts;
 
+#ifdef EAM_BOXED_TEXT
+extern textbox_style textbox_opts;
+#endif
+
 void default_arrow_style __PROTO((struct arrow_style_type *arrow));
+void apply_head_properties __PROTO((struct arrow_style_type *arrow_properties));
 
 void free_labels __PROTO((struct text_label *tl));
 
 void get_offsets __PROTO((struct text_label *this_label,
 	struct termentry *t, int *htic, int *vtic));
 void write_label __PROTO((unsigned int x, unsigned int y, struct text_label *label));
+int label_width __PROTO((const char *, int *));
 
 #endif /* GNUPLOT_GADGETS_H */
