@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: hidden3d.c,v 1.82.2.2 2012/01/04 05:17:29 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: hidden3d.c,v 1.99.2.5 2015/05/05 19:01:16 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - hidden3d.c */
@@ -313,10 +313,6 @@ static GP_INLINE double area2D __PROTO((p_vertex v1, p_vertex v2,
 					p_vertex v3));
 static void draw_vertex __PROTO((p_vertex v));
 static GP_INLINE void draw_edge __PROTO((p_edge e, p_vertex v1, p_vertex v2));
-static GP_INLINE void handle_edge_fragment __PROTO((long int edgenum,
-						    long int vnum1,
-						    long int vnum2,
-						    long int firstpoly));
 static int in_front __PROTO((long int edgenum,
 			     long int vnum1, long int vnum2,
 			     long int *firstpoly));
@@ -726,7 +722,7 @@ intersect_line_plane(p_vertex v1, p_vertex v2, t_plane p)
 	return 0;
     else {
 	double denominator = p[0]*(v1->x - v2->x) + p[1]*(v1->y - v2->y) + p[2]*(v1->z - v2->z);
-	return numerator/denominator;
+	return (denominator==0 ? (numerator>0?VERYLARGE:-VERYLARGE) : numerator/denominator);
     }
 }
 
@@ -741,7 +737,7 @@ intersect_line_line(p_vertex v1, p_vertex v2, p_vertex w1, p_vertex w2)
 	return 0;
     else {
 	double denominator = (w2->y - w1->y)*(v2->x - v1->x) - (w2->x - w1->x)*(v2->y - v1->y);
-	return numerator/denominator;
+	return (denominator==0 ? (numerator>0?VERYLARGE:-VERYLARGE) : numerator/denominator);
     }
 }
 
@@ -811,7 +807,7 @@ cover_point_poly(p_vertex v1, p_vertex v2, double u, p_polygon poly)
 static long int
 store_polygon(long vnum1, polygon_direction direction, long crvlen)
 {
-    long int v[POLY_NVERT];
+    long int v[POLY_NVERT] = {0};
     p_vertex v1, v2, v3;
     p_polygon p;
 
@@ -1099,11 +1095,13 @@ build_networks(struct surface_points *plots, int pcount)
 	 * plot styles are mapped to others, that are genuinely
 	 * available in 3d. */
 	switch (this_plot->plot_style) {
+	case PM3DSURFACE:
 	case LINESPOINTS:
 	case STEPS:
 	case FSTEPS:
 	case HISTEPS:
 	case LINES:
+	case SURFACEGRID:
 	    nv += nverts;
 	    ne += nverts - ncrvs;
 	    if (this_plot->has_grid_topology) {
@@ -1118,6 +1116,14 @@ build_networks(struct surface_points *plots, int pcount)
 	    nv += 2 * nverts;
 	    ne += nverts;
 	    break;
+	case DOTS:
+	    this_plot->lp_properties.flags |= LP_SHOW_POINTS;
+	    this_plot->lp_properties.p_type = -1;
+	case IMAGE:
+	case RGBIMAGE:
+	case RGBA_IMAGE:
+	    /* Ignore these */
+	    break;
 	case POINTSTYLE:
 	default:
 	    /* treat all remaining ones like 'points' */
@@ -1171,10 +1177,22 @@ build_networks(struct surface_points *plots, int pcount)
 	above = this_plot->hidden3d_top_linetype;
 	below = above + hiddenBacksideLinetypeOffset;
 
+	/* The "nosurface" flag is interpreted by hidden3d mode to mean */
+	/* "don't draw this surface".  I.e. draw only the contours.	*/
+	if (this_plot->opt_out_of_surface)
+	    above = below = LT_NODRAW;
+
 	/* This is a special flag indicating that the user specified an	*/
 	/* explicit surface color in the splot command.			*/
-	if (above == LT_SINGLECOLOR-1)
-	    above = below = LT_SINGLECOLOR;
+	if ((lp->flags & LP_EXPLICIT_COLOR))
+	    below = above;
+
+	/* We will not actually draw PM3D surfaces here, but their 	*/
+	/* edges can be used to calculate occlusion of lines, including */
+	/* the plot borders. (NB: the PM3D surface will _not_ be hidden */
+	/* by other non-PM3D surfaces.					*/
+	if (this_plot->plot_style == PM3DSURFACE)
+	    above = below = LT_NODRAW;
 
 	/* calculate the point symbol type: */
 	/* Assumes that upstream functions have made sure this is
@@ -1206,7 +1224,7 @@ build_networks(struct surface_points *plots, int pcount)
 		    long int thisvertex;
 		    struct coordinate labelpoint;
 
-		    lp->pointflag = 1; /* Labels can use the code for hidden points */
+		    lp->flags |= LP_SHOW_POINTS; /* Labels can use the code for hidden points */
 		    labelpoint.type = INRANGE;
 		    for (label = this_plot->labels->next; label != NULL; label = label->next) {
 			labelpoint.x = label->place.x;
@@ -1241,11 +1259,13 @@ build_networks(struct surface_points *plots, int pcount)
 		    }
 
 		    switch (this_plot->plot_style) {
+		    case PM3DSURFACE:
 		    case LINESPOINTS:
 		    case STEPS:
 		    case FSTEPS:
 		    case HISTEPS:
 		    case LINES:
+		    case SURFACEGRID:
 			if (previousvertex >= 0)
 			    store_edge(thisvertex, edir_west, 0, lp, above);
 			break;
@@ -1281,6 +1301,12 @@ build_networks(struct surface_points *plots, int pcount)
 			    store_edge(basevertex, edir_impulse, 0, lp, above);
 			break;
 
+		    case IMAGE:
+		    case RGBIMAGE:
+		    case RGBA_IMAGE:
+			/* Ignore these */
+			break;
+
 		    case POINTSTYLE:
 		    default:	/* treat all the others like 'points' */
 			store_edge(thisvertex, edir_point, crvlen, lp, above);
@@ -1326,11 +1352,13 @@ build_networks(struct surface_points *plots, int pcount)
 		    = -3;
 
 		switch (this_plot->plot_style) {
+		case PM3DSURFACE:
 		case LINESPOINTS:
 		case STEPS:
 		case FSTEPS:
 		case HISTEPS:
 		case LINES:
+		case SURFACEGRID:
 		    if (i > 0) {
 			/* not first point, so we might want to set up
 			 * the edge(s) to the left of this vertex */
@@ -1623,9 +1651,15 @@ static void
 draw_vertex(p_vertex v)
 {
     unsigned int x, y;
+    int p_type;
+
+    if (v->lp_style == NULL)
+	return;
+    
+    p_type = v->lp_style->p_type;
 
     TERMCOORD(v, x, y);
-    if (v->lp_style && v->lp_style->p_type >= -1 && !clip_point(x,y)) {
+    if ((p_type >= -1 || p_type == PT_CHARACTER) && !clip_point(x,y)) {
 	struct t_colorspec *tc = &(v->lp_style->pm3d_color);
 
 	if (v->label)  {
@@ -1641,9 +1675,9 @@ draw_vertex(p_vertex v)
 	    apply_pm3dcolor(tc, term);
 	}
 	else if (tc->type == TC_RGB && tc->lt == LT_COLORFROMCOLUMN)
-	    set_rgbcolor((int)v->real_z);
+	    set_rgbcolor_var((unsigned int)v->real_z);
 	else if (tc->type == TC_RGB)
-	    set_rgbcolor(tc->lt);
+	    set_rgbcolor_const(tc->lt);
 	else if (tc->type == TC_CB)
 	    set_color( cb2gray(v->real_z) );
 	else if (tc->type == TC_Z)
@@ -1654,7 +1688,10 @@ draw_vertex(p_vertex v)
 	    (term->pointsize)(pointsize * v->original->CRD_PTSIZE);
 #endif
 
-	(term->point)(x,y, v->lp_style->p_type);
+	if (p_type == PT_CHARACTER)
+	    (term->put_text)(x, y, (char *)(&(v->lp_style->p_char)));
+	else
+	    (term->point)(x,y, p_type);
 
 	/* vertex has been drawn --> flag it as done */
 	v->lp_style = NULL;
@@ -1688,7 +1725,7 @@ draw_edge(p_edge e, p_vertex v1, p_vertex v2)
     } else
 
     /* This handles explicit 'lc rgb' in the plot command */
-    if (color.type == TC_RGB && e->style == LT_SINGLECOLOR) {
+    if (color.type == TC_RGB && (lptemp.flags & LP_EXPLICIT_COLOR)) {
 	recolor = TRUE;
     } else
 
@@ -1726,14 +1763,24 @@ draw_edge(p_edge e, p_vertex v1, p_vertex v2)
 	lptemp.pm3d_color = color;
 	if (arrow)
 	    lptemp.p_type = e->style;
-#if 0 /* No longer correct; handled by load_linetype() above */
-	else
-	    lptemp.l_type = e->style;
-#endif
+    }
+
+    /* Only the original tip of an arrow should show an arrowhead */
+    /* FIXME:  Arrowhead lines are not themselves subject to hidden line removal */
+    if (arrow) {
+	if (e->v2 != v2-vlist && e->v1 != v1-vlist) {
+		lptemp.p_type = 0;
+	} else if (e->style == PT_BACKARROW) {
+	    if (e->v2 == v2-vlist && e->v1 != v1-vlist)
+		lptemp.p_type = 0;
+	} else {
+	    if (e->v1 == v1-vlist && e->v2 != v2-vlist)
+		lptemp.p_type = 0;
+	}
     }
 
     draw3d_line_unconditional(v1, v2, &lptemp, color);
-    if (e->lp->pointflag) {
+    if ((e->lp->flags & LP_SHOW_POINTS)) {
 	draw_vertex(v1);
 	draw_vertex(v2);
     }
@@ -1803,25 +1850,6 @@ area2D(p_vertex v1, p_vertex v2, p_vertex v3)
     return (dx12 * dy13 - dy12 * dx13);
 }
 
-/* Utility routine: takes an edge and makes a new one, which is a fragment
- * of the old one. The fragment inherits the line style and stuff of the
- * given edge; only the two new vertices are different. The new edge
- * is then passed to in_front, for recursive handling */
-/* HBB 20001108: Changed from edge pointer to edge index. Don't
- * allocate a fresh anymore, as this is no longer needed after the
- * change to in_front().  What remains of this function may no longer
- * be worth having. I.e. it can be replaced by a direct recursion call
- * of in_front(), sometime soon. */
-static GP_INLINE void
-handle_edge_fragment(long edgenum, long vnum1, long vnum2, long firstpoly)
-{
-#if !HIDDEN3D_QUADTREE
-    /* Avoid checking against the same polygon again. */
-    firstpoly = plist[firstpoly].next;
-#endif
-    in_front(edgenum, vnum1, vnum2, &firstpoly);
-}
-
 /*********************************************************************/
 /* The actual heart of all this: determines if edge at index 'edgenum'
  * of the elist is in_front of all the polygons, or not. If necessary,
@@ -1849,7 +1877,7 @@ in_front(
 
     coordval xmin, xmax;	/* all of these are for the edge */
     coordval ymin, ymax;
-    coordval zmin, zmax;
+    coordval zmin;
 #if HIDDEN3D_GRIDBOX
     unsigned int xextent;	/* extent bitmask in x direction */
     unsigned int yextent;	/* same, in y direction */
@@ -1895,7 +1923,7 @@ in_front(
 	}					\
 	vnum1 = v1 - vlist;			\
 	vnum2 = v2 - vlist;			\
-	zmax = v1->z;	zmin = v2->z;		\
+	zmin = v2->z;				\
 						\
 	if (v1->x > v2->x) {			\
 	    xmin = v2->x;	xmax = v1->x;	\
@@ -2026,7 +2054,7 @@ in_front(
 	     * it is removed.
 	     * 
 	     * This routine is general in the sense that the earlier tests
-	     * it are only need for speed.
+	     * are only needed for speed.
 	     * 
 	     * The following website illustrates geometrical concepts and
 	     * formulas:  http://local.wasp.uwa.edu.au/~pbourke/geometry/
@@ -2118,7 +2146,13 @@ in_front(
 				 * segment near end of an edge.  Simply ignore.
 				 */
 				if (newvert[1] != vnum1) {
-				    handle_edge_fragment(edgenum, newvert[1], vnum2, polynum);
+#if HIDDEN3D_QUADTREE
+				    in_front(edgenum, newvert[1], vnum2, &polynum);
+#else
+				    /* Avoid checking against the same polygon again. */
+				    in_front(edgenum, newvert[1], vnum2,
+						&plist[polynum].next);
+#endif
 				    setup_edge(vnum1, newvert[0]);
 				}
 				break;
@@ -2200,6 +2234,40 @@ draw_line_hidden(
 	droplast_dynarray(&vertices);
 }
 
+
+/* Externally callable function to draw a label, but hide it behind any
+ * visible occluding surfaces. */
+void
+draw_label_hidden(p_vertex v, struct lp_style_type *lp, int x, int y)
+{
+    long int thisvertex, edgenum, temp_pfirst;
+
+    /* If there is no surface to hide behind, just draw the label */
+    if (!polygons.end) {
+	write_label(x, y, v->label);
+	return;
+    }
+
+    nextfrom_dynarray(&vertices);
+    thisvertex = vertices.end - 1;
+    vlist[thisvertex] = *v;
+    vlist[thisvertex].lp_style = lp; /* Not sure this is necessary */
+
+    lp->flags |= LP_SHOW_POINTS; /* Labels can use the code for hidden points */
+
+    edgenum = make_edge(thisvertex, thisvertex, lp, lp->l_type, -1);
+
+    FPRINTF((stderr,"label: \"%s\" at [%d %d]  vertex %ld edge %ld\n", 
+    	v->label->text, x, y, thisvertex, edgenum));
+
+    temp_pfirst = pfirst;
+    in_front(edgenum, elist[edgenum].v1, elist[edgenum].v2, &temp_pfirst);
+
+    droplast_dynarray(&edges);
+    droplast_dynarray(&vertices);
+}
+
+
 /***********************************************************************
  * and, finally, the 'mother function' that uses all these lots of tools
  ***********************************************************************/
