@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: history.c,v 1.25.2.1 2010/08/06 01:23:06 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: history.c,v 1.31.2.1 2016/03/31 03:47:56 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - history.c */
@@ -39,17 +39,12 @@ static char *RCSid() { return RCSid("$Id: history.c,v 1.25.2.1 2010/08/06 01:23:
 #include "gp_hist.h"
 
 #include "alloc.h"
+#include "plot.h"
 #include "util.h"
 
-
-/* moved here from plot.c */
-#ifdef GNUPLOT_HISTORY
-# ifndef HISTORY_SIZE
-/* Can be overriden with the environment variable 'GNUPLOT_HISTORY_SIZE' */
-#  define HISTORY_SIZE 666
-# endif
-long int gnuplot_history_size = HISTORY_SIZE;
-#endif
+int gnuplot_history_size = HISTORY_SIZE;
+TBOOLEAN history_quiet = FALSE;
+TBOOLEAN history_full = FALSE;
 
 
 #if defined(READLINE) && !defined(HAVE_LIBREADLINE) && !defined(HAVE_LIBEDITLINE)
@@ -69,6 +64,7 @@ add_history(char *line)
     struct hist *entry;
 
     entry = history;
+    if (!history_full)
     while (entry != NULL) {
 	/* Don't store duplicate entries */
 	if (!strcmp(entry->line, line)) {
@@ -99,7 +95,11 @@ add_history(char *line)
 	entry = entry->prev;
     }				/* end of not-storing duplicated entries */
 
-#ifdef GNUPLOT_HISTORY
+#if (0)
+    /* This code block breaks "history !<number>" because it renumbers the  */
+    /* entries before retrieving the requested number.  Also it potentially */
+    /* truncates the list on every command contrary to the documentattion.  */
+
     /* limit size of history list to "gnuplot_history_size" */
     if (gnuplot_history_size != -1) {
 	while ((hist_count >= gnuplot_history_size) && (first_entry != NULL)) {
@@ -183,6 +183,7 @@ write_history_n(const int n, const char *filename, const char *mode)
     if (filename != NULL && filename[0]) {
 #ifdef PIPES
 	if (filename[0]=='|') {
+	    restrict_popen();
 	    out = popen(filename+1, "w");
 	    is_pipe = 1;
 	} else
@@ -254,6 +255,32 @@ read_history(char *filename)
     }
 }
 
+/* finds and returns a command from the history list by number */
+const char *
+history_find_by_number(int n)
+{
+    struct hist *entry = history;
+    int hist_entries = 0;
+    int index;
+
+    if (history == NULL || n <= 0)
+	return NULL;	/* no history yet */
+
+    /* count number of entries */
+    for (entry = history; entry->prev != NULL; entry = entry->prev)
+	hist_entries++;
+
+    if (n > hist_entries)
+	return NULL;
+
+    /* step backwards to the requested entry */
+    for (entry = history->prev, index = hist_entries; --index > 0; entry = entry->prev)
+	if (index + 1 == n)
+	    return entry->line;
+
+    return NULL;	/* should not happen */
+}
+
 
 /* finds and returns a command from the history which starts with <cmd>
  * (ignores leading spaces in <cmd>)
@@ -346,10 +373,7 @@ history_find_all(char *cmd)
  */
 
 void
-write_history_list(num, filename, mode)
-const int num;
-const char *const filename;
-const char *mode;
+write_history_list(const int num, const char *const filename, const char *mode)
 {
     const HIST_ENTRY *list_entry;
     FILE *out = stdout;
@@ -362,18 +386,20 @@ const char *mode;
         /* good filename given and not quiet */
 #ifdef PIPES
         if (filename[0]=='|') {
+	    restrict_popen();
             out = popen(filename+1, "w");
             is_pipe = 1;
-        } else {
+        } else
 #endif
+	{
             if (! (out = fopen(filename, mode) ) ) {
-                /* Fall back to 'stdout' */
                 int_warn(NO_CARET, "Cannot open file to save history, using standard output.\n");
                 out = stdout;
-            } else is_file = 1;
-#ifdef PIPES
+            } else {
+		is_file = 1;
+	    }
         }
-#endif
+
     } else if (filename && !filename[0])
         is_quiet = 1;
 
@@ -381,22 +407,20 @@ const char *mode;
      * For some reason the readline functions append_history() 
      * and write_history() do not work they way I thought they did...
      */
-    if (num > 0) {
+    if (num > 0)
         istart = history_length - num;
-        if (istart <= 0 || istart > history_length)
-            istart = 1;
-    } else istart = 1;
+    else
+	istart = 1;
+    if (istart <= 0 || istart > history_length)
+	istart = 1;
+
+    for (i = istart; (list_entry = history_get(i)); i++) {
+        /* don't add line numbers when writing to file to make file loadable */
+	if (!is_file && !is_quiet)
+	    fprintf(out, "%5i ", i + history_base - 1);
+	fprintf(out, "  %s\n", list_entry->line);
+    }
 
-        for (i = istart; (list_entry = history_get(i)); i++) {
-            /* don't add line numbers when writing to file to make file loadable */
-            if (is_file)
-                fprintf(out, "%s\n", list_entry->line);
-            else {
-                if (!is_quiet) fprintf(out, "%5i", i + history_base - 1);
-                fprintf(out, "  %s\n", list_entry->line);
-            }
-        }
-    /* close if something was opened */
 #ifdef PIPES
     if (is_pipe) pclose(out);
 #endif
@@ -405,22 +429,28 @@ const char *mode;
 
 /* This is the function getting called in command.c */
 void
-write_history_n(n, filename, mode)
-const int n;
-const char *filename;
-const char *mode;
+write_history_n(const int n, const char *filename, const char *mode)
 {
     write_history_list(n, filename, mode);
 }
 
+/* finds and returns a command from the history list by number */
+const char *
+history_find_by_number(int n)
+{
+    if (0 < n && n < history_length)
+	return history_get(n)->line;
+    else
+	return NULL;
+}
+
 /* finds and returns a command from the history which starts with <cmd>
  * Returns NULL if nothing found
  *
  * Peter Weilbacher, 28Jun2004
  */
 const char *
-history_find(cmd)
-char *cmd;
+history_find(char *cmd)
 {
     int len;
 
@@ -446,8 +476,7 @@ char *cmd;
  * Peter Weilbacher 28Jun2004
  */
 int
-history_find_all(cmd)
-char *cmd;
+history_find_all(char *cmd)
 {
     int len;
     int found;
@@ -464,15 +493,33 @@ char *cmd;
     /* Output matching history entries in chronological order (not backwards
      * so we have to start at the beginning of the history list.
      */
-    history_set_pos(0);
+#if defined(HAVE_LIBREADLINE)
+    found = history_set_pos(0);
+    if (found == -1) {
+        fprintf(stderr, "ERROR (history_find_all): could not rewind history\n");
+        return 0;
+    }
+#else /* HAVE_LIBEDITLINE */
+    /* libedit's history_set_pos() does not work properly,
+       so we manually go to oldest entry */
+    while (next_history());
+#endif
     do {
         found = history_search_prefix(cmd, 1); /* Anchored backward search for prefix */
         if (found == 0) {
             number++;
+#if defined(HAVE_LIBREADLINE)
             printf("%5i  %s\n", where_history() + history_base, current_history()->line);
             /* go one step back or you find always the same entry. */
             if (!history_set_pos(where_history() + 1))
                 break; /* finished if stepping didn't work */
+#else /* HAVE_LIBEDITLINE */
+            /* libedit's history indices are reversed wrt GNU readline */
+            printf("%5i  %s\n", history_length - where_history() + history_base, current_history()->line);
+            /* go one step back or you find always the same entry. */
+            if (!previous_history())
+                break; /* finished if stepping didn't work */
+#endif
         } /* (found == 0) */
     } while (found > -1);
 
