@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: misc.c,v 1.188.2.9 2015/12/29 20:33:35 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: misc.c,v 1.188.2.10 2016/08/16 19:35:19 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - misc.c */
@@ -1078,10 +1078,12 @@ lp_parse(struct lp_style_type *lp, lp_class destination_class, TBOOLEAN allow_po
 	    continue;
 	}
 
-	/* This is so that "set obj ... lw N fc <colorspec>" doesn't eat */
-	/* up the colorspec as a line property.  We need to parse it later */
-	/* as a _fill_ property */
-	if ((destination_class == LP_NOFILL)
+	/* This is so that "set obj ... lw N fc <colorspec>" doesn't eat up the
+	 * fc colorspec as a line property.  We need to parse it later as a
+	 * _fill_ property. Also prevents "plot ... fc <col1> fs <foo> lw <baz>"
+	 * from generating an error claiming redundant line properties.
+	 */
+	if ((destination_class == LP_NOFILL || destination_class == LP_ADHOC)
 	&&  (equals(c_token,"fc") || almost_equals(c_token,"fillc$olor")))
 	    break;
 
