@@ -46,6 +46,12 @@
 #include "QtGnuplotEvent.h"
 #include "QtGnuplotItems.h"
 
+extern "C" {
+#include "../mousecmn.h"
+// This is defined in term_api.h, but including it there fails to compile
+typedef enum t_fillstyle { FS_EMPTY, FS_SOLID, FS_PATTERN, FS_DEFAULT, FS_TRANSPARENT_SOLID, FS_TRANSPARENT_PATTERN } t_fillstyle;
+}
+
 #include <QtGui>
 #include <QDebug>
 
@@ -57,18 +63,26 @@ QtGnuplotScene::QtGnuplotScene(QtGnuplotEventHandler* eventHandler, QObject* par
 	m_lastModifierMask = 0;
 	m_textAngle = 0.;
 	m_textAlignment = Qt::AlignLeft;
+	m_textOffset = QPoint(10,10);
 	m_currentZ = 1.;
 	m_currentPointSize = 1.;
 	m_enhanced = 0;
 	m_currentPlotNumber = 0;
 	m_inKeySample = false;
 	m_preserve_visibility = false;
+	m_inTextBox = false;
+	m_currentPointsItem = new QtGnuplotPoints;
 
 	m_currentGroup.clear();
 
 	resetItems();
 }
 
+QtGnuplotScene::~QtGnuplotScene()
+{
+	delete m_currentPointsItem;
+}
+
 /////////////////////////////////////////////////
 // Gnuplot events
 
@@ -120,14 +134,30 @@ void QtGnuplotScene::flushCurrentPolygon()
 	}
 
 	clipPolygon(m_currentPolygon);
-	QPainterPath path;
-	path.addPolygon(m_currentPolygon);
-	QGraphicsPathItem *pathItem;
-	pathItem = addPath(path, m_currentPen, Qt::NoBrush);
-	pathItem->setZValue(m_currentZ++);
-	m_currentPolygon.clear();
 	if (!m_inKeySample)
-		m_currentGroup.append(pathItem);
+		m_currentPointsItem->addPolygon(m_currentPolygon, m_currentPen);
+	else
+	{
+		flushCurrentPointsItem();
+		// Including the polygon in a path is necessary to avoid closing the polygon
+		QPainterPath path;
+		path.addPolygon(m_currentPolygon);
+		QGraphicsPathItem *pathItem;
+		pathItem = addPath(path, m_currentPen, Qt::NoBrush);
+		pathItem->setZValue(m_currentZ++);
+	}
+	m_currentPolygon.clear();
+}
+
+void QtGnuplotScene::flushCurrentPointsItem()
+{
+	if (m_currentPointsItem->isEmpty())
+		return;
+
+	m_currentPointsItem->setZValue(m_currentZ++);
+	addItem(m_currentPointsItem);
+	m_currentGroup.append(m_currentPointsItem);
+	m_currentPointsItem = new QtGnuplotPoints();
 }
 
 void QtGnuplotScene::update_key_box(const QRectF rect)
@@ -178,7 +208,10 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 		QPointF point; in >> point;
 		m_currentPolygon << point;
 		if (m_inKeySample)
+		{
+			flushCurrentPointsItem();
 			update_key_box( QRectF(point, QSize(0,1)) );
+		}
 	}
 	else if (type == GEPenColor)
 	{
@@ -187,13 +220,13 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 	}
 	else if (type == GEBackgroundColor)
 	{
-		m_currentPen.setColor(m_widget->m_backgroundColor);
+		m_currentPen.setColor(m_widget->backgroundColor());
 	}
 	else if (type == GEPenStyle)
 	{
 		int style; in >> style;
 		m_currentPen.setStyle(Qt::PenStyle(style));
-		if (m_widget->m_rounded) {
+		if (m_widget->rounded()) {
 			m_currentPen.setJoinStyle(Qt::RoundJoin);
 			m_currentPen.setCapStyle(Qt::RoundCap);
 		} else {
@@ -201,6 +234,11 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 			m_currentPen.setCapStyle(Qt::FlatCap);
 		}
 	}
+	else if (type == GEDashPattern)
+	{
+		QVector<qreal> dashpattern; in >> dashpattern;
+		m_currentPen.setDashPattern(dashpattern);
+	}
 	else if (type == GEPointSize)
 		in >> m_currentPointSize;
 	else if (type == GETextAngle)
@@ -213,6 +251,7 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 	}
 	else if (type == GEFillBox)
 	{
+		flushCurrentPointsItem();
 		QRect rect; in >> rect;
 		QGraphicsRectItem *rectItem;
 		rectItem = addRect(rect, Qt::NoPen, m_currentBrush);
@@ -225,15 +264,20 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 	else if (type == GEFilledPolygon)
 	{
 		QPolygonF polygon; in >> polygon;
-		QPen pen = Qt::NoPen;
-		if (m_currentBrush.style() == Qt::SolidPattern)
-			pen = m_currentBrush.color();
-		clipPolygon(polygon, false);
-		QGraphicsPolygonItem *path;
-		path = addPolygon(polygon, pen, m_currentBrush);
-		path->setZValue(m_currentZ++);
+
 		if (!m_inKeySample)
-			m_currentGroup.append(path);
+			m_currentPointsItem->addFilledPolygon(clipPolygon(polygon, false), m_currentBrush);
+		else
+		{
+			flushCurrentPointsItem();
+			QPen pen = Qt::NoPen;
+			if (m_currentBrush.style() == Qt::SolidPattern)
+				pen = m_currentBrush.color();
+			clipPolygon(polygon, false);
+			QGraphicsPolygonItem *path;
+			path = addPolygon(polygon, pen, m_currentBrush);
+			path->setZValue(m_currentZ++);
+		}
 	}
 	else if (type == GEBrushStyle)
 	{
@@ -251,44 +295,75 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 		int size        ; in >> size;
 		m_font.setFamily(fontName);
 		m_font.setPointSize(size);
+		m_font.setStyleStrategy(QFont::ForceOutline);	// pcf fonts die if rotated
 	}
 	else if (type == GEPoint)
 	{
 		QPointF point; in >> point;
 		int style    ; in >> style;
-		QtGnuplotPoint* pointItem = new QtGnuplotPoint(style, m_currentPointSize, m_currentPen.color());
-		pointItem->setPos(clipPoint(point));
-		pointItem->setZValue(m_currentZ++);
-		addItem(pointItem);
-		if (m_inKeySample)
-			update_key_box( QRectF(point, QSize(2,2)) );
+
+		// Make sure point symbols are always drawn with a solid line
+		m_currentPen.setStyle(Qt::SolidLine);
+
+		// Append the point to a points item to speed-up drawing
+		if (!m_inKeySample)
+			m_currentPointsItem->addPoint(clipPoint(point), style, m_currentPointSize, m_currentPen);
 		else
-			m_currentGroup.append(pointItem);
+		{
+			flushCurrentPointsItem();
+			QtGnuplotPoint* pointItem = new QtGnuplotPoint(style, m_currentPointSize, m_currentPen);
+			pointItem->setPos(clipPoint(point));
+			pointItem->setZValue(m_currentZ++);
+			addItem(pointItem);
+			update_key_box( QRectF(point, QSize(2,2)) );
+		}
+
+		// Create a hypertext label that will become visible on mouseover.
+		// The Z offset is a kludge to force the label into the foreground.
+		if (!m_currentHypertext.isEmpty()) {
+			QGraphicsTextItem* textItem = addText(m_currentHypertext, m_font);
+			textItem->setPos(point + m_textOffset);
+			textItem->setZValue(m_currentZ+10000);
+			textItem->setVisible(false);
+			m_hypertextList.append(textItem);
+			m_currentHypertext.clear();
+		}
 	}
 	else if (type == GEPutText)
 	{
+		flushCurrentPointsItem();
 		QPoint point; in >> point;
 		QString text; in >> text;
 		QGraphicsTextItem* textItem = addText(text, m_font);
 		textItem->setDefaultTextColor(m_currentPen.color());
 		positionText(textItem, point);
-		if (m_inKeySample) {
-			QRectF rect = textItem->boundingRect();
-			qreal fullheight = rect.height();
-			if (m_textAlignment == Qt::AlignRight)
-				rect.moveBottomRight(point);
-			else
-				rect.moveBottomLeft(point);
-			rect.adjust(0, fullheight/1.8, 0, fullheight/4);
+
+		QRectF rect = textItem->boundingRect();
+		if (m_textAlignment == Qt::AlignCenter) {
+			rect.moveCenter(point);
+			rect.moveBottom(point.y());
+		} else if (m_textAlignment == Qt::AlignRight)
+			rect.moveBottomRight(point);
+		else
+			rect.moveBottomLeft(point);
+		rect.adjust(0, rect.height()*0.67, 0, rect.height()*0.25);
+
+		if (m_inKeySample)
 			update_key_box(rect);
-		} else {
+		else
 			m_currentGroup.append(textItem);
-		}
+#ifdef EAM_BOXED_TEXT
+		if (m_inTextBox)
+			m_currentTextBox |= rect;
+#endif
 	}
 	else if (type == GEEnhancedFlush)
 	{
+		flushCurrentPointsItem();
 		QString fontName; in >> fontName;
 		double fontSize ; in >> fontSize;
+		int fontStyle   ; in >> fontStyle;	// really QFont::Style
+		int fontWeight  ; in >> fontWeight;	// really QFont::Weight
 		double base     ; in >> base;
 		bool widthFlag  ; in >> widthFlag;
 		bool showFlag   ; in >> showFlag;
@@ -296,30 +371,56 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 		QString text    ; in >> text;
 		if (m_enhanced == 0)
 			m_enhanced = new QtGnuplotEnhanced();
-		m_enhanced->addText(fontName, fontSize, base, widthFlag, showFlag,
+		m_enhanced->addText(fontName, fontSize, 
+				    (QFont::Style)fontStyle, (QFont::Weight)fontWeight,
+				    base, widthFlag, showFlag,
 		                    overprint, text, m_currentPen.color());
 	}
 	else if (type == GEEnhancedFinish)
 	{
+		flushCurrentPointsItem();
 		QPoint point; in >> point;
-		addItem(m_enhanced);
 		positionText(m_enhanced, point);
+		m_enhanced->setZValue(m_currentZ++);
+		addItem(m_enhanced);
+
+		QRectF rect = m_enhanced->boundingRect();
+		if (m_textAlignment == Qt::AlignCenter) {
+			rect.moveCenter(point);
+			rect.moveBottom(point.y());
+		} else if (m_textAlignment == Qt::AlignRight)
+			rect.moveBottomRight(point);
+		else
+			rect.moveBottomLeft(point);
+		rect.adjust(-m_currentPen.width()*2, rect.height()/2, 
+			    m_currentPen.width()*2, rect.height()/2);
+
+		if (m_inKeySample)
+			update_key_box(rect);
+		else
+			m_currentGroup.append(m_enhanced);
+#ifdef EAM_BOXED_TEXT
+		if (m_inTextBox)
+			m_currentTextBox |= rect;
+#endif
 		m_enhanced = 0;
 	}
 	else if (type == GEImage)
 	{
-		QPoint p0; in >> p0;
-		QPoint p1; in >> p1;
-		QPoint p2; in >> p2;
-		QPoint p3; in >> p3;
+		flushCurrentPointsItem();
+		QPointF p0; in >> p0;
+		QPointF p1; in >> p1;
+		QPointF p2; in >> p2;
+		QPointF p3; in >> p3;
 		QImage image; in >> image;
-		QPixmap pixmap = QPixmap::fromImage(image);
-		QGraphicsPixmapItem* item = addPixmap(pixmap);
+		QSize size(p1.x() - p0.x(), p1.y() - p0.y());
+		QRectF clipRect(p2 - p0, p3 - p0);
+		QPixmap pixmap = QPixmap::fromImage(image).scaled(size, Qt::IgnoreAspectRatio, Qt::FastTransformation);
+		QtGnuplotClippedPixmap* item = new QtGnuplotClippedPixmap(clipRect, pixmap);
+		addItem(item);
 		item->setZValue(m_currentZ++);
 		item->setPos(p0);
-		QPointF size = p1 - p0;
-		item->scale(size.x()/pixmap.width(), size.y()/pixmap.height());
-		/// @todo clipping
+		m_currentGroup.append(item);
 	}
 	else if (type == GEZoomStart)
 	{
@@ -371,10 +472,10 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 		for (int i = 0; i < 4; i++)
 			in >> m_axisValid[i] >> m_axisMin[i] >> m_axisLower[i] >> m_axisScale[i] >> m_axisLog[i];
 	}
-	else if (type == GEPlotNumber) 
+	else if (type == GEAfterPlot) 
 	{
-		int newPlotNumber;  in >> newPlotNumber;
-		if (newPlotNumber == 0 && m_currentPlotNumber > 0) {
+		flushCurrentPointsItem();
+		if (m_currentPlotNumber >= m_plot_group.count()) {
 			// End of previous plot, create group holding the accumulated elements
 			QGraphicsItemGroup *newgroup;
 			newgroup = createItemGroup(m_currentGroup);
@@ -383,28 +484,155 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 			if (0 < m_currentPlotNumber && m_currentPlotNumber <= m_key_boxes.count())
 				newgroup->setVisible( !(m_key_boxes[m_currentPlotNumber-1].ishidden()) );
 			// Store it in an ordered list so we can toggle it by index
-			if (m_currentPlotNumber >= m_plot_group.count())
-				m_plot_group.insert(m_currentPlotNumber, newgroup);
-			else
-				m_plot_group.replace(m_currentPlotNumber-1, newgroup);
+			m_plot_group.insert(m_currentPlotNumber, newgroup);
 		} 
-		else
-		{
+		m_currentPlotNumber = 0;
+	}
+	else if (type == GEPlotNumber) 
+	{
+		flushCurrentPointsItem();
+		int newPlotNumber;  in >> newPlotNumber;
+		if (newPlotNumber >= m_plot_group.count()) {
 			// Initialize list of elements for next group
 			m_currentGroup.clear();
-		}
+		} 
+		// Otherwise we are making a second pass through the same plots
+		// (currently used only to draw key samples on an opaque key box)
 		m_currentPlotNumber = newPlotNumber;
 	}
+	else if (type == GEModPlots)
+	{
+		int i = m_key_boxes.count();
+		unsigned int ops_i;
+		int plotno;
+		in >> ops_i;
+		in >> plotno;
+		enum QtGnuplotModPlots ops = (enum QtGnuplotModPlots) ops_i;
+
+		/* FIXME: This shouldn't happen, but it does. */
+		/* Failure to reset lists after multiplot??   */
+		if (i > m_plot_group.count())
+		    i = m_plot_group.count();
+
+		while (i-- > 0) {
+			/* Does operation only affect a single plot? */
+			if (plotno >= 0 && i != plotno)
+				continue;
+
+			bool isVisible = m_plot_group[i]->isVisible();
+
+			switch (ops) {
+			case QTMODPLOTS_INVERT_VISIBILITIES:
+				isVisible = !isVisible;
+				break;
+			case QTMODPLOTS_SET_VISIBLE:
+				isVisible = true;
+				break;
+			case QTMODPLOTS_SET_INVISIBLE:
+				isVisible = false;
+				break;
+			}
+
+			m_plot_group[i]->setVisible(isVisible);
+			m_key_boxes[i].setHidden(!isVisible);
+		}
+	}
 	else if (type == GELayer)
 	{
+		flushCurrentPointsItem();
 		int layer; in >> layer;
-		if (layer == QTLAYER_BEGIN_KEYSAMPLE) m_inKeySample = true;
-		if (layer == QTLAYER_END_KEYSAMPLE) m_inKeySample = false;
 		if (layer == QTLAYER_BEFORE_ZOOM) m_preserve_visibility = true;
+		if (layer == QTLAYER_BEGIN_KEYSAMPLE) m_inKeySample = true;
+		if (layer == QTLAYER_END_KEYSAMPLE)
+		{
+			m_inKeySample = false;
+
+			// FIXME: this catches mislabeled opaque keyboxes in multiplot mode
+			if (m_currentPlotNumber > m_key_boxes.length()) {
+				return;
+			}
+			// Draw an invisible grey rectangle in the key box.
+			// It will be set to visible if the plot is toggled off.
+			QtGnuplotKeybox *keybox = &m_key_boxes[m_currentPlotNumber-1];
+			m_currentBrush.setColor(Qt::lightGray);
+			m_currentBrush.setStyle(Qt::Dense4Pattern);
+			QGraphicsRectItem *statusBox = addRect(*keybox, Qt::NoPen, m_currentBrush);
+			statusBox->setZValue(m_currentZ-1);
+			keybox->showStatus(statusBox);
+		}
+	}
+	else if (type == GEHypertext)
+	{
+		m_currentHypertext.clear();
+		in >> m_currentHypertext;
+	}
+#ifdef EAM_BOXED_TEXT
+	else if (type == GETextBox)
+	{
+		flushCurrentPointsItem();
+		QPointF point; in >> point;
+		int option; in >> option;
+		QGraphicsRectItem *rectItem;
+		QRectF outline;
+
+		/* Must match the definition in ../term_api.h */
+		enum t_textbox_options {
+			TEXTBOX_INIT = 0,
+			TEXTBOX_OUTLINE,
+			TEXTBOX_BACKGROUNDFILL,
+			TEXTBOX_MARGINS,
+			TEXTBOX_FINISH,
+			TEXTBOX_GREY
+		};
+
+		switch (option) {
+		case TEXTBOX_INIT:
+			/* Initialize bounding box */
+			m_currentTextBox = QRectF(point, point);
+			m_inTextBox = true;
+			break;
+		case TEXTBOX_OUTLINE:
+			/* Stroke bounding box */
+			outline = m_currentTextBox.adjusted(
+				 -m_textMargin.x(), -m_textMargin.y(),
+				  m_textMargin.x(), m_textMargin.y());
+			rectItem = addRect(outline, m_currentPen, Qt::NoBrush);
+			rectItem->setZValue(m_currentZ++);
+			m_currentGroup.append(rectItem);
+			m_inTextBox = false;
+			break;
+		case TEXTBOX_BACKGROUNDFILL:
+			/* Fill bounding box */
+			m_currentBrush.setColor(m_widget->backgroundColor());
+			m_currentBrush.setStyle(Qt::SolidPattern);
+			outline = m_currentTextBox.adjusted(
+				 -m_textMargin.x(), -m_textMargin.y(),
+				  m_textMargin.x(), m_textMargin.y());
+			rectItem = addRect(outline, Qt::NoPen, m_currentBrush);
+			rectItem->setZValue(m_currentZ++);
+			m_currentGroup.append(rectItem);
+			m_inTextBox = false;
+			break;
+		case TEXTBOX_MARGINS:
+			/* Set margins of bounding box */
+			m_textMargin = point;
+			m_textMargin *= QFontMetrics(m_font).averageCharWidth();
+			break;
+		}
+	}
+#endif
+	else if (type == GEFontMetricRequest)
+	{
+		QFontMetrics metrics(m_font);
+		int par1 = (metrics.ascent() + metrics.descent());
+		int par2 = metrics.width("0123456789")/10.;
+		m_eventHandler->postTermEvent(GE_fontprops, 0, 0, par1, par2, m_widget);
 	}
 	else if (type == GEDone)
-		m_eventHandler->postTermEvent(GE_plotdone, 0, 0, 0, 0, 0);
-		/// @todo m_id;//qDebug() << "Done !" << items().size();
+	{
+		flushCurrentPointsItem();
+		m_eventHandler->postTermEvent(GE_plotdone, 0, 0, 0, 0, m_widget);
+	}
 	else
 		swallowEvent(type, in);
 }
@@ -412,6 +640,8 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 void QtGnuplotScene::resetItems()
 {
 	clear();
+	delete m_currentPointsItem;
+	m_currentPointsItem = new QtGnuplotPoints;
 
 	m_currentZ = 1.;
 
@@ -431,9 +661,15 @@ void QtGnuplotScene::resetItems()
 	int i = m_key_boxes.count();
 	while (i-- > 0) {
 		m_key_boxes[i].setSize( QSizeF(0,0) );
+		m_key_boxes[i].resetStatus();
 		if (!m_preserve_visibility)
 			m_key_boxes[i].setHidden(false);
 	}
+
+	m_hypertextList.clear();
+	m_hypertextList.append(addRect(QRect(), QPen(QColor(0, 0, 0, 100)), QBrush(QColor(225, 225, 225, 200))));
+	m_hypertextList[0]->setVisible(false);
+
 	m_plot_group.clear();	// Memory leak?  Destroy groups first?
 }
 
@@ -452,9 +688,9 @@ void QtGnuplotScene::setBrushStyle(int style)
 
 	QColor color = m_currentPen.color();
 
-	if ((fillstyle == FS_TRANSPARENT_SOLID) || ((fillstyle == FS_SOLID) && (fillpar == 100)))
+	if (fillstyle == FS_TRANSPARENT_SOLID)
 		color.setAlphaF(double(fillpar)/100.);
-	else if (fillstyle == FS_SOLID)
+	else if (fillstyle == FS_SOLID && (fillpar < 100))
 	{
 		double fact  = double(100 - fillpar)/100.;
 		double factc = 1. - fact;
@@ -470,7 +706,7 @@ void QtGnuplotScene::setBrushStyle(int style)
 		/// @todo color & transparent. See other terms
 		m_currentBrush.setStyle(QtGnuplot::brushes[abs(fillpar) % 8]);
 	else if (fillstyle == FS_EMPTY) // fill with background plot->color
-		color = Qt::white;
+		color = m_widget->backgroundColor();
 
 	m_currentBrush.setColor(color);
 }
@@ -488,9 +724,8 @@ void QtGnuplotScene::positionText(QGraphicsItem* item, const QPoint& point)
 	else if (m_textAlignment == Qt::AlignCenter)
 		cx = (item->boundingRect().right() + item->boundingRect().left())/2.;
 
-	item->translate(cx, cy);
-	item->rotate(-m_textAngle);
-	item->translate(-cx, -cy);
+	item->setTransformOriginPoint(cx, cy);
+	item->setRotation(-m_textAngle);
 	item->setPos(point.x() - cx, point.y() - cy);
 }
 
@@ -527,7 +762,7 @@ void QtGnuplotScene::updateModifiers()
 	if (modifierMask != m_lastModifierMask)
 	{
 		m_lastModifierMask = modifierMask;
-		m_eventHandler->postTermEvent(GE_modifier, 0, 0, modifierMask, 0, 0); /// @todo m_id
+		m_eventHandler->postTermEvent(GE_modifier, 0, 0, modifierMask, 0, m_widget);
 	}
 }
 
@@ -543,7 +778,7 @@ void QtGnuplotScene::mousePressEvent(QGraphicsSceneMouseEvent* event)
 
 	m_eventHandler->postTermEvent(GE_buttonpress, 
 			int(event->scenePos().x()), int(event->scenePos().y()), 
-			button, 0, 0); /// @todo m_id
+			button, 0, m_widget);
 	QGraphicsScene::mousePressEvent(event);
 }
 
@@ -581,7 +816,24 @@ void QtGnuplotScene::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
 		m_lineTo->setLine(line);
 	}
 
-	m_eventHandler->postTermEvent(GE_motion, int(event->scenePos().x()), int(event->scenePos().y()), 0, 0, 0); /// @todo m_id
+	// The first item in m_hypertextList is always a background rectangle for the text
+	int i = m_hypertextList.count();
+	bool hit = false;
+	while (i-- > 1) {
+		if (!hit && ((m_hypertextList[i]->pos() - m_textOffset) 
+				- m_lastMousePos).manhattanLength() <= 5) {
+			hit = true;
+			m_hypertextList[i]->setVisible(true);
+			((QGraphicsRectItem *)m_hypertextList[0])->setRect(m_hypertextList[i]->boundingRect());
+			m_hypertextList[0]->setPos(m_hypertextList[i]->pos());
+			m_hypertextList[0]->setZValue(m_hypertextList[i]->zValue()-1);
+		} else {
+			m_hypertextList[i]->setVisible(false);
+		}
+	}
+	m_hypertextList[0]->setVisible(hit);
+
+	m_eventHandler->postTermEvent(GE_motion, int(event->scenePos().x()), int(event->scenePos().y()), 0, 0, m_widget);
 	QGraphicsScene::mouseMoveEvent(event);
 }
 
@@ -595,30 +847,28 @@ void QtGnuplotScene::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
 	else if (event->button()== Qt::MidButton)   button = 2;
 	else if (event->button()== Qt::RightButton) button = 3;
 
-	// EAM FIXME - If the press/release events get out of order or if we see
-	// a very fast double-click then the following call causes the program
-	// to error out during event processing.  Also note that I can't find
-	// where the timers are initialized on entry - maybe a problem there?
-	// Finally, why do we need or want a timer at all?
-	int time = m_watches[button].elapsed();
-	if (time > 300) // purely empical work-around
-	if (m_eventHandler->postTermEvent(GE_buttonrelease, int(event->scenePos().x()),
-	    int(event->scenePos().y()), button, time, 0))
-		m_watches[button].start();
+	qint64 time = 301;	/* Only used the first time in, when timer not yet running */
+	if (m_watches[button].isValid())
+		time = m_watches[button].elapsed();
+	m_eventHandler->postTermEvent(GE_buttonrelease,
+		int(event->scenePos().x()), int(event->scenePos().y()), button, time, m_widget);
+	m_watches[button].start();
 
-	/* Check for click in one of the keysample boxes */
-	int i = m_key_boxes.count();
-	while (i-- > 0) {
-		if (m_key_boxes[i].contains(m_lastMousePos)) {
-			if (m_plot_group[i]->isVisible()) {
-				m_plot_group[i]->setVisible(false);
-				m_key_boxes[i].setHidden(true);
-			} else {
-				m_plot_group[i]->setVisible(true);
-				m_key_boxes[i].setHidden(false);
-			}
-			break;
-		}
+	/* Check for left click in one of the keysample boxes */
+	if (button == 1) {
+	    int n = m_key_boxes.count();
+	    for (int i = 0; i < n; i++) {
+		    if (m_key_boxes[i].contains(m_lastMousePos)) {
+			    if (m_plot_group[i]->isVisible()) {
+				    m_plot_group[i]->setVisible(false);
+				    m_key_boxes[i].setHidden(true);
+			    } else {
+				    m_plot_group[i]->setVisible(true);
+				    m_key_boxes[i].setHidden(false);
+			    }
+			    break;
+		    }
+	    }
 	}
 
 	QGraphicsScene::mouseReleaseEvent(event);
@@ -632,12 +882,12 @@ void QtGnuplotScene::wheelEvent(QGraphicsSceneWheelEvent* event)
 		// 6 = scroll left, 7 = scroll right
 		m_eventHandler->postTermEvent(GE_buttonpress,
 			int(event->scenePos().x()), int(event->scenePos().y()), 
-			event->delta() > 0 ? 6 : 7, 0, 0);
+			event->delta() > 0 ? 6 : 7, 0, m_widget);
 	} else { /* if (event->orientation() == Qt::Vertical) */
 		// 4 = scroll up, 5 = scroll down
 		m_eventHandler->postTermEvent(GE_buttonpress,
 			int(event->scenePos().x()), int(event->scenePos().y()), 
-			event->delta() > 0 ? 4 : 5, 0, 0);
+			event->delta() > 0 ? 4 : 5, 0, m_widget);
 	} 
 }
 
@@ -646,6 +896,7 @@ void QtGnuplotScene::keyPressEvent(QKeyEvent* event)
 	updateModifiers();
 
 	int key = -1;
+	int live;
 
 	/// @todo quit on 'q' or Ctrl+'q'
 
@@ -662,12 +913,19 @@ void QtGnuplotScene::keyPressEvent(QKeyEvent* event)
 			case Qt::Key_F4       : key = GP_KP_F4       ; break;
 			case Qt::Key_Insert   : key = GP_KP_Insert   ; break;
 			case Qt::Key_End      : key = GP_KP_End      ; break;
+			case Qt::Key_Home     : key = GP_KP_Home     ; break;
+#ifdef __APPLE__
+			case Qt::Key_Down     : key = GP_Down        ; break;
+			case Qt::Key_Left     : key = GP_Left        ; break;
+			case Qt::Key_Right    : key = GP_Right       ; break;
+			case Qt::Key_Up       : key = GP_Up          ; break;
+#else
 			case Qt::Key_Down     : key = GP_KP_Down     ; break;
-			case Qt::Key_PageDown : key = GP_KP_Page_Down; break;
 			case Qt::Key_Left     : key = GP_KP_Left     ; break;
 			case Qt::Key_Right    : key = GP_KP_Right    ; break;
-			case Qt::Key_Home     : key = GP_KP_Home     ; break;
 			case Qt::Key_Up       : key = GP_KP_Up       ; break;
+#endif
+			case Qt::Key_PageDown : key = GP_KP_Page_Down; break;
 			case Qt::Key_PageUp   : key = GP_KP_Page_Up  ; break;
 			case Qt::Key_Delete   : key = GP_KP_Delete   ; break;
 			case Qt::Key_Equal    : key = GP_KP_Equal    ; break;
@@ -691,7 +949,7 @@ void QtGnuplotScene::keyPressEvent(QKeyEvent* event)
 	// ASCII keys
 	else if ((event->key() <= 0xff) && (!event->text().isEmpty()))
 		// event->key() does not respect the case
-		key = event->text()[0].toAscii();
+		key = event->text()[0].toLatin1();
 	// Special keys
 	else
 		switch (event->key())
@@ -728,7 +986,31 @@ void QtGnuplotScene::keyPressEvent(QKeyEvent* event)
 		}
 
 	if (key >= 0)
-		m_eventHandler->postTermEvent(GE_keypress, int(m_lastMousePos.x()), int(m_lastMousePos.y()), key, 0, 0); /// @todo m_id
+		live = m_eventHandler->postTermEvent(GE_keypress,
+			int(m_lastMousePos.x()), int(m_lastMousePos.y()), key, 0, m_widget);
+	else
+		live = true;
+
+	// Key handling in persist mode 
+	// !live means (I think!) that we are in persist mode
+	if (!live) {
+		switch (key) {
+		default:
+			break;
+		case 'i':
+			int i = m_key_boxes.count();
+			/* FIXME: This shouldn't happen, but it does. */
+			if (i > m_plot_group.count())
+			    i = m_plot_group.count();
+			while (i-- > 0) {
+				bool isVisible = m_plot_group[i]->isVisible();
+				isVisible = !isVisible;
+				m_plot_group[i]->setVisible(isVisible);
+				m_key_boxes[i].setHidden(!isVisible);
+			}
+			break;
+		}
+	}
 
 	QGraphicsScene::keyPressEvent(event);
 }
