@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: readline.c,v 1.58 2011/09/04 12:01:37 markisch Exp $"); }
+static char *RCSid() { return RCSid("$Id: readline.c,v 1.62.2.4 2016/08/23 00:23:34 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - readline.c */
@@ -77,9 +77,10 @@ getc_wrapper(FILE* fp /* should be stdin, supplied by readline */)
     int c;
 
     while (1) {
+	errno = 0;
 #ifdef USE_MOUSE
 	if (term && term->waitforinput && interactive) {
-	    c = term->waitforinput();
+	    c = term->waitforinput(0);
 	}
 	else
 #endif
@@ -248,11 +249,11 @@ static int ansi_getc __PROTO((void));
 #  define TEXTGNUPLOT 0xf0
 #  ifdef WGP_CONSOLE
 #   define special_getc() win_getch()
-static char win_getch __PROTO((void));
+static int win_getch(void);
 #  else
-#  define special_getc() msdos_getch()
+#   define special_getc() msdos_getch()
 #  endif /* WGP_CONSOLE */
-static char msdos_getch __PROTO((void));	/* HBB 980308: PROTO'ed it */
+static int msdos_getch(void);
 #  define DEL_ERASES_CURRENT_CHAR
 # endif				/* _Windows */
 
@@ -265,7 +266,7 @@ static char msdos_getch __PROTO((void));	/* HBB 980308: PROTO'ed it */
 #   include <conio.h>
 #  endif			/* __EMX__ */
 #  define special_getc() msdos_getch()
-static char msdos_getch();
+static int msdos_getch();
 #  define DEL_ERASES_CURRENT_CHAR
 # endif				/* MSDOS */
 
@@ -276,15 +277,15 @@ static char msdos_getch();
 #  undef special_getc()
 # endif				/* special_getc */
 # define special_getc() os2_getch()
-static char msdos_getch __PROTO((void));	/* HBB 980308: PROTO'ed it */
-static char os2_getch __PROTO((void));
+static int msdos_getch(void);
+static int os2_getch(void);
 #  define DEL_ERASES_CURRENT_CHAR
 #endif /* OS2 */
 
 
 /* initial size and increment of input line length */
 #define MAXBUF	1024
-#define BACKSPACE 0x08   /* ^H */
+#define BACKSPACE '\b'   /* ^H */
 #define SPACE	' '
 #define NEWLINE	'\n'
 
@@ -361,12 +362,12 @@ user_puts(char *str)
 /* EAM FIXME
  * This test is intended to determine if the current character, of which
  * we have only seen the first byte so far, will require twice the width
- * of an ascii character.  The test catches glyphs above unicode 0x3000, 
+ * of an ascii character.  The test catches glyphs above unicode 0x3000,
  * which is roughly the set of CJK characters.
  * It should be replaced with a more accurate test.
  */
 static int
-mbwidth(char *c) 
+mbwidth(char *c)
 {
     switch (encoding) {
 
@@ -375,7 +376,7 @@ mbwidth(char *c)
 	wchar_t wc;
 	if (mbtowc(&wc, c, MB_CUR_MAX) < 0)
 	    return 1;
-	else 
+	else
 	    return wcwidth(wc);
 #else
 	return ((unsigned char)(*c) >= 0xe3 ? 2 : 1);
@@ -402,17 +403,14 @@ char_seqlen()
 {
     int i;
 
-    switch (encoding) {
-
-    case S_ENC_UTF8:
+    if (S_ENC_UTF8 == encoding) {
 	i = cur_pos;
 	do {i++;}
-	    while ((cur_line[i] & 0xc0) != 0xc0 
-	        && (cur_line[i] & 0x80) != 0
-		&& i < max_pos);
+	while (((cur_line[i] & 0xc0) != 0xc0)
+	       && ((cur_line[i] & 0x80) != 0)
+	       && (i < max_pos));
 	return (i - cur_pos);
-
-    default:
+    } else {
 	return 1;
     }
 }
@@ -426,23 +424,24 @@ backspace()
 {
     int seqlen;
 
-    switch (encoding) {
-
-    case S_ENC_UTF8:
+    if (S_ENC_UTF8 == encoding) {
 	seqlen = 0;
-	do {cur_pos--; seqlen++;}
-	    while ((cur_line[cur_pos] & 0xc0) != 0xc0 
-	        && (cur_line[cur_pos] & 0x80) != 0
-		&& cur_pos > 0);
-    
-	if ((cur_line[cur_pos] & 0xc0) == 0xc0
-	||  isprint(cur_line[cur_pos]))
+	do {
+	    cur_pos--;
+	    seqlen++;
+	} while (((cur_line[cur_pos] & 0xc0) != 0xc0)
+	         && ((cur_line[cur_pos] & 0x80) != 0)
+		 && (cur_pos > 0)
+	        );
+
+	if (   ((cur_line[cur_pos] & 0xc0) == 0xc0)
+	    || isprint((unsigned char)cur_line[cur_pos])
+	   )
 	    user_putc(BACKSPACE);
 	if (isdoublewidth(cur_pos))
 	    user_putc(BACKSPACE);
 	return seqlen;
-
-    default:
+    } else {
 	cur_pos--;
 	user_putc(BACKSPACE);
 	return 1;
@@ -549,20 +548,23 @@ fn_completion(size_t anchor_pos, int direction)
 	    n_completions = 0;
 	    completion_idx = 0;
 	}
-	
+
 	/* extract path to complete */
 	start = cur_line + anchor_pos;
 	if (anchor_pos > 0) {
 	    /* first, look for a quote to start the string */
-	    for ( ; start > cur_line; start--) {
+	    for ( ; start >= cur_line; start--) {
 	        if ((*start == '"') || (*start == '\'')) {
 		    start++;
+		    /* handle pipe commands */
+		    if ((*start == '<') || (*start == '|'))
+			start++;
 		    break;
 		}
 	    }
-	    /* if not found, search for a space instead */
-	    if (start == cur_line) {
-		for (start = cur_line + anchor_pos ; start > cur_line; start--) {
+	    /* if not found, search for a space or a system command '!' instead */
+	    if (start <= cur_line) {
+		for (start = cur_line + anchor_pos; start >= cur_line; start--) {
 		    if ((*start == ' ') || (*start == '!')) {
 			start++;
 			break;
@@ -604,7 +606,8 @@ fn_completion(size_t anchor_pos, int direction)
 	    struct dirent * entry;
 	    while ((entry = readdir(dir)) != NULL) {
 		/* ignore files and directories starting with a dot */
-		if (entry->d_name[0] == '.') continue; 
+		if (entry->d_name[0] == '.')
+		    continue;
 
 		/* skip entries which don't match */
 		if (nlen > 0)
@@ -619,10 +622,10 @@ fn_completion(size_t anchor_pos, int direction)
 	    closedir(dir);
 	    free(search);
 	    if (name) free(name);
-            if (n_completions > 0)
-	        return completions[0];
-            else 
-                return NULL;
+	    if (n_completions > 0)
+		return completions[0];
+	    else
+		return NULL;
 	}
 	free(search);
 	if (name) free(name);
@@ -686,8 +689,8 @@ tab_completion(TBOOLEAN forward)
 
     /* insert completion string */
     if (max_pos > (last_tab_pos - last_completion_len))
-	memmove(cur_line + last_tab_pos + completion_len, 
-		cur_line + last_tab_pos + last_completion_len, 
+	memmove(cur_line + last_tab_pos + completion_len,
+		cur_line + last_tab_pos + last_completion_len,
 		max_pos  - last_tab_pos - last_completion_len);
     memcpy(cur_line + last_tab_pos, completion, completion_len);
     max_pos += completion_len - last_completion_len;
@@ -741,9 +744,9 @@ readline(const char *prompt)
 	 * and all leading 8bit characters.
 	 */
 	if ((isprint(cur_char)
-	      || (((cur_char & 0x80) != 0) && (cur_char != EOF)))
-	    && (cur_char != 0x09) /* TAB is a printable character in some locales */
-	    ) {
+	     || (((cur_char & 0x80) != 0) && (cur_char != EOF)))
+	    && (cur_char != '\t') /* TAB is a printable character in some locales */
+	   ) {
 	    size_t i;
 
 	    if (max_pos + 1 >= line_len) {
@@ -766,7 +769,7 @@ readline(const char *prompt)
 			fix_line(); /* Normal ascii character */
 		    else if ((cur_char & 0xc0) == 0xc0)
 			; /* start of a multibyte sequence. */
-		    else if (((cur_char & 0xc0) == 0x80) && 
+		    else if (((cur_char & 0xc0) == 0x80) &&
 			 ((unsigned char)(cur_line[cur_pos-2]) >= 0xe0))
 			; /* second byte of a >2 byte sequence */
 		    else {
@@ -847,6 +850,9 @@ readline(const char *prompt)
 	    case 011:		/* ^I / TAB */
 		tab_completion(TRUE); /* next tab completion */
 		break;
+	    case 034:		/* remapped by wtext.c or ansi_getc from Shift-Tab */
+		tab_completion(FALSE); /* previous tab completion */
+		break;
 #endif
 	    case 013:		/* ^K */
 		clear_eoline(prompt);
@@ -963,7 +969,6 @@ fix_line()
     i = cur_pos;
     for (cur_pos = max_pos; cur_pos > i; )
 	backspace();
-
 }
 
 /* redraw the entire line, putting the cursor where it belongs */
@@ -1085,7 +1090,7 @@ ansi_getc()
 
 #ifdef USE_MOUSE
     if (term && term->waitforinput && interactive)
-	c = term->waitforinput();
+	c = term->waitforinput(0);
     else
 #endif
     c = getc(stdin);
@@ -1113,6 +1118,9 @@ ansi_getc()
 	    case 'H':		/* home key */
 		c = 001;
 		break;
+	    case 'Z':		/* shift-tab key */
+		c = 034;	/* FS: non-standard! */
+		break;
 	    case '3':		/* DEL can be <esc>[3~ */
 		getc(stdin);	/* eat the ~ */
 		c = 023;	/* DC3 ^S NB: non-standard!! */
@@ -1126,46 +1134,49 @@ ansi_getc()
 #if defined(MSDOS) || defined(_Windows) || defined(OS2)
 
 #ifdef WGP_CONSOLE
-static char
+static int
 win_getch()
 {
     if (term && term->waitforinput)
-        return term->waitforinput();
+        return term->waitforinput(0);
     else
         return ConsoleGetch();
 }
 #endif
 
 /* Convert Arrow keystrokes to Control characters: */
-static char
+static int
 msdos_getch()
 {
 	char c;
-	
+
 #ifdef DJGPP
     int ch = getkey();
     c = (ch & 0xff00) ? 0 : ch & 0xff;
 #elif defined (OS2)
     c = getc(stdin);
 #else /* not OS2, not DJGPP*/
-# if defined (_Windows) && defined (USE_MOUSE)
+# if defined (USE_MOUSE)
     if (term && term->waitforinput && interactive)
-	c = term->waitforinput();
+	c = term->waitforinput(0);
     else
-# endif /* not _Windows && not USE_MOUSE */
+# endif /* not USE_MOUSE */
     c = getch();
 #endif /* not DJGPP, not OS2 */
 
     if (c == 0) {
 #ifdef DJGPP
 	c = ch & 0xff;
-#else /* not DJGPP */
-# ifdef OS2
+#elif defined(OS2)
 	c = getc(stdin);
-# else				/* not OS2 */
+#else /* not OS2, not DJGPP */
+# if defined (USE_MOUSE)
+    if (term && term->waitforinput && interactive)
+	c = term->waitforinput(0);
+    else
+# endif /* not USE_MOUSE */
 	c = getch();		/* Get the extended code. */
-# endif				/* OS2 */
-#endif /* not DJGPP */
+#endif /* not DJGPP, not OS2 */
 
 	switch (c) {
 	case 75:		/* Left Arrow. */
@@ -1206,7 +1217,7 @@ msdos_getch()
 #ifdef OS2
 /* We need to call different procedures, dependent on the
    session type: VIO/window or an (XFree86) xterm */
-static char
+static int
 os2_getch() {
   static int IsXterm = 0;
   static int init = 0;
