@@ -1,5 +1,5 @@
 /*
- * $Id: term.h,v 1.42 2009/01/09 00:30:35 sfeam Exp $
+ * $Id: term.h,v 1.66.2.2 2016/05/06 23:08:32 broeker Exp $
  */
 
 /* GNUPLOT - term.h */
@@ -39,7 +39,7 @@
  *   Edit this file depending on the set of terminals you wish to support.
  * Comment out the terminal types that you don't want or don't have, and
  * uncomment those that you want included. Be aware that some terminal
- * types (eg, SUN, UNIXPLOT) will require changes in the makefile
+ * types (eg, SUN) will require changes in the makefile
  * LIBS definition.
  */
 
@@ -97,18 +97,6 @@
 /* Platform dependent part                                                  */
 /****************************************************************************/
 
-/* Amiga */
-#ifdef AMIGA
-# include "amiga.trm"
-#endif
-
-
-/* Apple Macintosh */
-#ifdef _Macintosh
-# include "mac.trm"
-#endif
-
-
 /* BeOS */
 #ifdef __BEOS__
 # include "be.trm"
@@ -117,7 +105,7 @@
 
 /****************************************************************************/
 /* MS-DOS and Windows */
-#if defined(MSDOS) || defined(_Windows) || defined(DOS386)
+#if defined(MSDOS) || defined(_Windows)
 
 /* MSDOS with emx-gcc compiler */
 # if defined(MSDOS) && defined(__EMX__)
@@ -131,11 +119,6 @@
 #  include "djsvga.trm"
 # endif
 
-/* MSDOS with Zortech-C++ Compiler */
-# ifdef __ZTC__
-#  include "fg.trm"
-# endif
-
 /* All other Compilers */
 # ifndef _Windows
 #  ifdef PC
@@ -168,7 +151,7 @@
 */
 
 /* Apple Mac OS X */
-#ifdef HAVE_LIBAQUATERM
+#ifdef HAVE_FRAMEWORK_AQUATERM
 /* support for AquaTerm.app */
 # include "aquaterm.trm"
 #endif
@@ -188,16 +171,6 @@
 /* Terminals for various Unix platforms                                    */
 /***************************************************************************/
 
-/* Apollo Graphics Primitive Resource */
-#ifdef APOLLO
-/* with resizeable windows */
-# include "apollo.trm"
-#  ifdef GPR
-/* with fixed windows */
-#   include "gpr.trm"
-#  endif
-#endif /* Apollo */
-
 /* Linux VGA */
 #ifdef LINUXVGA
 # include "linux.trm"
@@ -208,23 +181,6 @@
 # endif
 #endif /* LINUXVGA */
 
-/* MGR Window system */
-#ifdef MGR
-# include "mgr.trm"
-#endif
-
-/* Redwood Graphics Interface Protocol UNIPLEX */
-/* Metafile, requires POSIX */
-#ifdef RGIP
-# include "rgip.trm"
-#endif
-
-
-/* SCO CGI drivers */
-#ifdef SCO
-# include "cgi.trm"
-#endif
-
 /* SunView */
 #ifdef SUN
 # include "sun.trm"
@@ -236,31 +192,19 @@
 # include "vws.trm"
 #endif
 
-/* AT&T Unix-PC */
-#ifdef UNIXPC
-# include "unixpc.trm"
-#endif
-
 /****************************************************************************/
-/* Terminals not relevant for MSDOS, MS-Windows, Amiga             */
-#if !(defined(MSDOS) || defined(_Windows) || defined(AMIGA))
-
-/* AED 512 and AED 767 graphics terminals */
-/* # include "aed.trm" */
-
-# if defined(UNIXPLOT) || defined(GNUGRAPH)
-#  ifdef GNUGRAPH
-#   include "gnugraph.trm"
-#  else
-#   include "unixplot.trm"
-#  endif			/* !GNUGRAPH */
-# endif				/* UNIXPLOT || GNUGRAPH */
+/* Terminals not relevant for MSDOS, MS-Windows */
+#if !(defined(MSDOS) || defined(_Windows))
 
 /* gpic for groff */
+#ifdef HAVE_GPIC
 # include "gpic.trm"
+#endif
 
 /* REGIS graphics language */
+#ifdef VMS
 # include "regis.trm"
+#endif
 
 /* Tektronix 4106, 4107, 4109 and 420x terminals */
 # include "t410x.trm"
@@ -269,9 +213,13 @@
 # include "tek.trm"
 
 
-#endif /* !MSDOS && !_Windows && !AMIGA */
+#endif /* !MSDOS && !_Windows */
 /****************************************************************************/
 
+#ifdef GP_ENH_EST
+#include "estimate.trm"
+#endif
+
 
 /****************************************************************************/
 /* These terminals can be used on any system */
@@ -286,6 +234,9 @@
 /* #include "ai.trm" */
 
 /* HTML Canvas terminal */
+#if (defined(HAVE_GD_PNG) || defined(HAVE_CAIROPDF))
+# include "write_png_image.c"
+#endif
 #include "canvas.trm"
 
 /* Computer Graphics Metafile (eg ms office) */
@@ -302,6 +253,11 @@
 /* dumb terminal */
 #include "dumb.trm"
 
+/* caca: color ascii art terminal using libcaca */
+#ifdef HAVE_LIBCACA
+# include "caca.trm"
+#endif
+
 /* DXF format for use with AutoCad (Release 10.x) */
 #include "dxf.trm"
 
@@ -319,35 +275,37 @@
 /* geographical info system */
 /* #include "grass.trm" */
 
-/* HP2623A and probably others */
-#include "hp26.trm"
+/* HP2623A "ET head" 1980 era graphics terminal */
+/* #include "hp26.trm" */
 
 /* HP2647 and 2648 */
-#include "hp2648.trm"
-
-/* HP DeskJet 500 C */
-#include "hp500c.trm"
+/* #include "hp2648.trm" */
 
 /* HP7475, HP7220 plotters, and (hopefully) lots of others */
 #include "hpgl.trm"
 
+#ifndef NO_BITMAP_SUPPORT
+/* HP DeskJet 500 C */
+#include "hp500c.trm"
+
 /* HP Laserjet II */
 #include "hpljii.trm"
 
 /* HP PrintJet */
 #include "hppj.trm"
 
+#endif /* NO_BITMAP_SUPPORT */
+
 /* Imagen laser printers */
-#include "imagen.trm"
+/* #include "imagen.trm" */
 
 /* Kyocera Prescribe printer */
 /* #include "kyo.trm" */
 
 /* Frame Maker MIF 3.00 format driver */
+#ifdef HAVE_MIF
 #include "mif.trm"
-
-/* portable bit map */
-#include "pbm.trm"
+#endif
 
 /* Adobe Portable Document Format (PDF) */
 /* NOTE THAT PDF REQUIRES A SEPARATE LIBRARY : see term/pdf.trm */
@@ -376,8 +334,10 @@
 /* tcl/tk with perl extensions */
 #include "tkcanvas.trm"
 
-/* Vectrix 384 printer, also Tandy colour */
-/* #include "v384.trm" */
+#ifndef NO_BITMAP_SUPPORT
+
+/* portable bit map */
+#include "pbm.trm"
 
 /* wire printers */
 /* Epson LX-800, Star NL-10, NX-1000 and lots of others */
@@ -402,6 +362,7 @@
 /* the common driver file for all of these */
 #include "epson.trm"
 
+#endif /* NO_BITMAP_SUPPORT */
 
 /* TeX related terminals */
 #define EMTEX
@@ -433,14 +394,13 @@
 /* METAPOST */
 #include "metapost.trm"
 
+/* ConTeXt */
+#include "context.trm"
+
 #ifdef USE_GGI_DRIVER
 # include "ggi.trm"
 #endif
 
-#ifdef GP_ENH_EST
-#include "estimate.trm"
-#endif
-
 /* WXWIDGETS */
 #ifdef WXWIDGETS
 # include "wxt.trm"
@@ -454,4 +414,8 @@
 #include "lua.trm"
 #endif
 
+#ifdef QTTERM
+# include "qt.trm"
+#endif
+
 #endif /* !SHORT_TERMLIST */
