@@ -1,5 +1,5 @@
 #ifndef lint
-static char *RCSid() { return RCSid("$Id: util.c,v 1.99.2.6 2013/02/08 23:01:39 sfeam Exp $"); }
+static char *RCSid() { return RCSid("$Id: util.c,v 1.128.2.5 2016/08/19 16:14:08 sfeam Exp $"); }
 #endif
 
 /* GNUPLOT - util.c */
@@ -42,8 +42,10 @@ static char *RCSid() { return RCSid("$Id: util.c,v 1.99.2.6 2013/02/08 23:01:39
 #include "internal.h"		/* for eval_reset_after_error */
 #include "misc.h"
 #include "plot.h"
-#include "term_api.h"		/* for term_end_plot() used by graph_error() */
-#include "variable.h" /* For locale handling */
+#include "term_api.h"		/* for term_end_plot() used by graph_error(), also to detect enhanced mode */
+#include "variable.h"		/* For locale handling */
+#include "setshow.h"		/* for conv_text() */
+#include "tabulate.h"		/* for table_mode */
 
 #if defined(HAVE_DIRENT_H)
 # include <sys/types.h>
@@ -51,6 +53,9 @@ static char *RCSid() { return RCSid("$Id: util.c,v 1.99.2.6 2013/02/08 23:01:39
 #elif defined(_Windows)
 # include <windows.h>
 #endif
+#if defined(__MSC__) || defined (__WATCOMC__)
+# include <io.h>
+#endif
 
 /* Exported (set-table) variables */
 
@@ -60,6 +65,10 @@ char *decimalsign = NULL;
 /* degree sign.  Defaults to UTF-8 but will be changed to match encoding */
 char degree_sign[8] = "°";
 
+/* minus sign (encoding-specific string) */
+const char *minus_sign = NULL;
+TBOOLEAN use_minus_sign = FALSE;
+
 /* Holds the name of the current LC_NUMERIC as set by "set decimal locale" */
 char *numeric_locale = NULL;
 
@@ -72,6 +81,8 @@ const char *current_prompt = NULL; /* to be set by read_line() */
 
 static void mant_exp __PROTO((double, double, TBOOLEAN, double *, int *, const char *));
 static void parse_sq __PROTO((char *));
+static TBOOLEAN utf8_getmore __PROTO((unsigned long * wch, const char **str, int nbytes));
+static char *utf8_strchrn __PROTO((const char *s, int N));
 
 /*
  * equals() compares string value of token number t_num with str[], and
@@ -82,7 +93,7 @@ equals(int t_num, const char *str)
 {
     int i;
 
-    if (t_num >= num_tokens)	/* safer to test here than to trust all callers */
+    if (t_num < 0 || t_num >= num_tokens)	/* safer to test here than to trust all callers */
 	return (FALSE);
     if (!token[t_num].is_token)
 	return (FALSE);		/* must be a value--can't be equal */
@@ -105,13 +116,17 @@ almost_equals(int t_num, const char *str)
 {
     int i;
     int after = 0;
-    int start = token[t_num].start_index;
-    int length = token[t_num].length;
+    int start, length;
 
+    if (t_num < 0 || t_num >= num_tokens)	/* safer to test here than to trust all callers */
+	return FALSE;
     if (!str)
 	return FALSE;
     if (!token[t_num].is_token)
 	return FALSE;		/* must be a value--can't be equal */
+
+    start = token[t_num].start_index;
+    length = token[t_num].length;
     for (i = 0; i < length + after; i++) {
 	if (str[i] != gp_input_line[start + i]) {
 	    if (str[i] != '$')
@@ -171,9 +186,9 @@ isanumber(int t_num)
 int
 isletter(int t_num)
 {
+    unsigned char c = gp_input_line[token[t_num].start_index];
     return (token[t_num].is_token &&
-	    ((isalpha((unsigned char) gp_input_line[token[t_num].start_index])) ||
-	     (gp_input_line[token[t_num].start_index] == '_')));
+	    (isalpha(c) || (c == '_') || ALLOWED_8BITVAR(c)));
 }
 
 
@@ -214,9 +229,16 @@ is_definition(int t_num)
 void
 copy_str(char *str, int t_num, int max)
 {
-    int i = 0;
-    int start = token[t_num].start_index;
-    int count = token[t_num].length;
+    int i, start, count;
+
+    if (t_num >= num_tokens) {
+	*str = NUL;
+	return;
+    }
+
+    i = 0;
+    start = token[t_num].start_index;
+    count = token[t_num].length;
 
     if (count >= max) {
 	count = max - 1;
@@ -237,6 +259,12 @@ token_len(int t_num)
     return (size_t)(token[t_num].length);
 }
 
+#ifdef NEXT
+/*
+ * quote_str() no longer has any callers in the core code.
+ * However, it is called by the next/openstep terminal.
+ */
+
 /*
  * quote_str() does the same thing as copy_str, except it ignores the
  *   quotes at both ends.  This seems redundant, but is done for
@@ -265,7 +293,7 @@ quote_str(char *str, int t_num, int max)
     else
 	parse_sq(str);
 }
-
+#endif
 
 /*
  * capture() copies into str[] the part of gp_input_line[] which lies between
@@ -499,9 +527,6 @@ mant_exp(
 
 /*}}} */
 
-#ifndef HAVE_SNPRINTF
-# define snprintf(str,len,fmt,arg) sprintf(str,fmt,arg)
-#endif
 
 /*{{{  gprintf */
 /* extended s(n)printf */
@@ -530,23 +555,29 @@ gprintf(
     TBOOLEAN seen_mantissa = FALSE; /* remember if mantissa was already output */
     double stored_power_base = 0;   /* base for the last mantissa output*/
     int stored_power = 0;	/* power matching the mantissa output earlier */
-    TBOOLEAN got_hash = FALSE;				   
+    TBOOLEAN got_hash = FALSE;
 
-    char *dest = &tempdest[0];
+    char *dest  = &tempdest[0];
     char *limit = &tempdest[MAX_LINE_LEN];
+    static double log10_of_1024; /* to avoid excess precision comparison in check of connection %b -- %B */
+    
+    log10_of_1024 = log10(1024);
+    
 #define remaining_space (size_t)(limit-dest)
 
     *dest = '\0';
 
     set_numeric_locale();
 
+    /* Oct 2013 - default format is now expected to be "%h" */
+    if (((term->flags & TERM_IS_LATEX)) && !strcmp(format, DEF_FORMAT))
+	format = DEF_FORMAT_LATEX;
+
     for (;;) {
 	/*{{{  copy to dest until % */
 	while (*format != '%')
 	    if (!(*dest++ = *format++) || (remaining_space == 0)) {
-		safe_strncpy(outstring,tempdest,count);
-		reset_numeric_locale();
-		return;		/* end of format */
+		goto done;
 	    }
 	/*}}} */
 
@@ -566,7 +597,7 @@ gprintf(
 	    format++;
 	    got_hash = TRUE;
 	}
-	/* dont put isdigit first since sideeffect in macro is bad */
+	/* dont put isdigit first since side effect in macro is bad */
 	while (*++format == '.' || isdigit((unsigned char) *format)
 	       || *format == '-' || *format == '+' || *format == ' '
 	       || *format == '\'')
@@ -604,6 +635,83 @@ gprintf(
 	    t[1] = 0;
 	    snprintf(dest, remaining_space, temp, x);
 	    break;
+	case 'h':
+	case 'H':
+	    /* g/G with enhanced formating (if applicable) */
+	    t[0] = (*format == 'h') ? 'g' : 'G';
+	    t[1] = 0;
+
+	    if ((term->flags & (TERM_ENHANCED_TEXT | TERM_IS_LATEX)) == 0) {
+		/* Not enhanced, not latex, just print it */
+		snprintf(dest, remaining_space, temp, x);
+
+	    } else if (table_mode) {
+		/* Tabular output should contain no markup */
+		snprintf(dest, remaining_space, temp, x);
+
+	    } else {
+		/* in enhanced mode -- convert E/e to x10^{foo} or *10^{foo} */
+#define LOCAL_BUFFER_SIZE 256
+		char tmp[LOCAL_BUFFER_SIZE];
+		char tmp2[LOCAL_BUFFER_SIZE];
+		int i,j;
+		TBOOLEAN bracket_flag = FALSE;
+		snprintf(tmp, 240, temp, x); /* magic number alert: why 240? */
+		for (i=j=0; tmp[i] && (i < LOCAL_BUFFER_SIZE); i++) {
+		    if (tmp[i]=='E' || tmp[i]=='e') {
+			if ((term-> flags & TERM_IS_LATEX)) {
+			    if (*format == 'h') {
+				strcpy(&tmp2[j], "\\times");
+				j+= 6;
+			    } else {
+				strcpy(&tmp2[j], "\\cdot");
+				j+= 5;
+			    }
+			} else switch (encoding) {
+			    case S_ENC_UTF8:
+				strcpy(&tmp2[j], "\xc3\x97"); /* UTF character '×' */
+				j+= 2;
+				break;
+			    case S_ENC_CP1252:
+				tmp2[j++] = (*format=='h') ? 0xd7 : 0xb7;
+				break;
+			    case S_ENC_ISO8859_1:
+			    case S_ENC_ISO8859_2:
+			    case S_ENC_ISO8859_9:
+			    case S_ENC_ISO8859_15:
+				tmp2[j++] = (*format=='h') ? 0xd7 : '*';
+				break;
+			    default:
+				tmp2[j++] = (*format=='h') ? 'x' : '*';
+				break;
+			}
+
+			strcpy(&tmp2[j], "10^{");
+			j += 4;
+			bracket_flag = TRUE;
+
+			/* Skip + and leading 0 in exponent */
+			i++; /* skip E */
+			if (tmp[i] == '+')
+			    i++;
+			else if (tmp[i] == '-') /* copy sign */
+			    tmp2[j++] = tmp[i++];
+			while (tmp[i] == '0')
+			    i++;
+			i--; /* undo following loop increment */
+		    } else {
+			tmp2[j++] = tmp[i];
+		    }
+		}
+		if (bracket_flag)
+		    tmp2[j++] = '}';
+		tmp2[j] = '\0';
+		strncpy(dest, tmp2, remaining_space);
+#undef LOCAL_BUFFER_SIZE
+	    }
+
+	    break;
+
 	    /*}}} */
 	    /*{{{  l --- mantissa to current log base */
 	case 'l':
@@ -657,7 +765,7 @@ gprintf(
 
 		t[0] = 'f';
 		t[1] = 0;
-		stored_power_base = log10(1024);
+		stored_power_base = log10_of_1024;
 		mant_exp(stored_power_base, x, FALSE, &mantissa,
 				&stored_power, temp);
 		seen_mantissa = TRUE;
@@ -672,14 +780,16 @@ gprintf(
 
 		t[0] = 'd';
 		t[1] = 0;
-		if (seen_mantissa)
-		    if (stored_power_base == log10_base)
+		if (seen_mantissa) {
+		    if (stored_power_base == log10_base) {
 			power = stored_power;
-		    else
+		    } else {
 			int_error(NO_CARET, "Format character mismatch: %%L is only valid with %%l");
-		else
+		    }
+		} else {
 		    stored_power_base = log10_base;
 		    mant_exp(log10_base, x, FALSE, NULL, &power, "%.0f");
+		}
 		snprintf(dest, remaining_space, temp, power);
 		break;
 	    }
@@ -691,13 +801,15 @@ gprintf(
 
 		t[0] = 'd';
 		t[1] = 0;
-		if (seen_mantissa)
-		    if (stored_power_base == 1.0)
+		if (seen_mantissa) {
+		    if (stored_power_base == 1.0) {
 			power = stored_power;
-		    else
+		    } else {
 			int_error(NO_CARET, "Format character mismatch: %%T is only valid with %%t");
-		else
+		    }
+		} else {
 		    mant_exp(1.0, x, FALSE, NULL, &power, "%.0f");
+		}
 		snprintf(dest, remaining_space, temp, power);
 		break;
 	    }
@@ -709,13 +821,15 @@ gprintf(
 
 		t[0] = 'd';
 		t[1] = 0;
-		if (seen_mantissa)
-		    if (stored_power_base == 1.0)
+		if (seen_mantissa) {
+		    if (stored_power_base == 1.0) {
 			power = stored_power;
-		    else
+		    } else {
 			int_error(NO_CARET, "Format character mismatch: %%S is only valid with %%s");
-		else
+		    }
+		} else {
 		    mant_exp(1.0, x, TRUE, NULL, &power, "%.0f");
+		}
 		snprintf(dest, remaining_space, temp, power);
 		break;
 	    }
@@ -727,21 +841,17 @@ gprintf(
 
 		t[0] = 'c';
 		t[1] = 0;
-		if (seen_mantissa)
-		    if (stored_power_base == 1.0)
+		if (seen_mantissa) {
+		    if (stored_power_base == 1.0) {
 			power = stored_power;
-		    else
+		    } else {
 			int_error(NO_CARET, "Format character mismatch: %%c is only valid with %%s");
-		else
+		    }
+		} else {
 		    mant_exp(1.0, x, TRUE, NULL, &power, "%.0f");
+		}
 
 		if (power >= -24 && power <= 24) {
-		    /* -18 -> 0, 0 -> 6, +18 -> 12, ... */
-		    /* HBB 20010121: avoid division of -ve ints! */
-		    power = (power + 24) / 3;
-		    snprintf(dest, remaining_space, temp, "yzafpnum kMGTPEZY"[power]);
-		} else {
-		    /* please extend the range ! */
 		    /* name  power   name  power
 		       -------------------------
 		       yocto  -24    yotta  24
@@ -752,7 +862,12 @@ gprintf(
 		       nano    -9    Giga    9
 		       micro   -6    Mega    6
 		       milli   -3    kilo    3   */
-
+		    /* -18 -> 0, 0 -> 6, +18 -> 12, ... */
+		    /* HBB 20010121: avoid division of -ve ints! */
+		    power = (power + 24) / 3;
+		    snprintf(dest, remaining_space, temp, "yzafpnum kMGTPEZY"[power]);
+		} else {
+		    /* please extend the range ! */
 		    /* fall back to simple exponential */
 		    snprintf(dest, remaining_space, "e%+02d", power);
 		}
@@ -766,14 +881,16 @@ gprintf(
 
 		t[0] = 'c';
 		t[1] = 'i';
-		t[2] = 0;
-		if (seen_mantissa)
-		    if (stored_power_base == log10(1024))
+		t[2] = '\0';
+		if (seen_mantissa) {
+		    if (stored_power_base == log10_of_1024) {
 			power = stored_power;
-		    else
+		    } else {
 			int_error(NO_CARET, "Format character mismatch: %%B is only valid with %%b");
-		else
-			mant_exp(log10(1024), x, FALSE, NULL, &power, "%.0f");
+		    }
+		} else {
+			mant_exp(log10_of_1024, x, FALSE, NULL, &power, "%.0f");
+		}
 
 		if (power > 0 && power <= 8) {
 		    /* name  power
@@ -792,7 +909,9 @@ gprintf(
 		    snprintf(dest, remaining_space, "x2^{%d}Yi", power-8);
 		} else if (power < 0) {
 		    snprintf(dest, remaining_space, "x2^{%d}", power*10);
-		}
+		} else {
+                    snprintf(dest, remaining_space, "  ");
+                }
 
 		break;
 	    }
@@ -811,38 +930,79 @@ gprintf(
 	   int_error(NO_CARET, "Bad format character");
 	} /* switch */
 	/*}}} */
-	
+
 	if (got_hash && (format != strpbrk(format,"oeEfFgG"))) {
 	   reset_numeric_locale();
 	   int_error(NO_CARET, "Bad format character");
 	}
 
-    /* change decimal `.' to the actual entry in decimalsign */
+    /* change decimal '.' to the actual entry in decimalsign */
 	if (decimalsign != NULL) {
-	    char *dotpos1 = dest, *dotpos2;
+	    char *dotpos1 = dest;
+	    char *dotpos2;
 	    size_t newlength = strlen(decimalsign);
-	    int dot;
 
 	    /* dot is the default decimalsign we will be replacing */
-	    dot = *get_decimal_locale();
+	    int dot = *get_decimal_locale();
 
 	    /* replace every dot by the contents of decimalsign */
 	    while ((dotpos2 = strchr(dotpos1,dot)) != NULL) {
-		size_t taillength = strlen(dotpos2);
-
-		dotpos1 = dotpos2 + newlength;
-		/* test if the new value for dest would be too long */
-		if (dotpos1 - dest + taillength > count)
-		    int_error(NO_CARET,
-			      "format too long due to long decimalsign string");
-		/* move tail end of string out of the way */
-		memmove(dotpos1, dotpos2 + 1, taillength);
-		/* insert decimalsign */
-		memcpy(dotpos2, decimalsign, newlength);
+		if (newlength == 1) {	/* The normal case */
+		    *dotpos2 = *decimalsign;
+		    dotpos1++;
+		} else {		/* Some multi-byte decimal marker */
+		    size_t taillength = strlen(dotpos2);
+		    dotpos1 = dotpos2 + newlength;
+		    if (dotpos1 + taillength > limit)
+			int_error(NO_CARET,
+				  "format too long due to decimalsign string");
+		    /* move tail end of string out of the way */
+		    memmove(dotpos1, dotpos2 + 1, taillength);
+		    /* insert decimalsign */
+		    memcpy(dotpos2, decimalsign, newlength);
+		}
+	    }
+	}
+
+    /* EXPERIMENTAL
+     * Some people prefer a "real" minus sign to the hyphen that standard
+     * formatted input and output both use.  Unlike decimal signs, there is
+     * no internationalization mechanism to specify this preference.
+     * This code replaces all hyphens with the character string specified by
+     * 'set minus_sign "..."'   typically unicode character U+2212 "−".
+     * Use at your own risk.  Should be OK for graphical output, but text output
+     * will not be readable by standard formatted input routines.
+     */
+	if (use_minus_sign		/* set minussign */
+	    && minus_sign		/* current encoding provides one */
+	    && !table_mode		/* not used inside "set table" */
+	    && !(term->flags & TERM_IS_LATEX)	/* but LaTeX doesn't want it */
+	   ) {
+
+	    char *dotpos1 = dest;
+	    char *dotpos2;
+	    size_t newlength = strlen(minus_sign);
+
+	    /* dot is the default hyphen we will be replacing */
+	    int dot = '-';
+
+	    /* replace every dot by the contents of minus_sign */
+	    while ((dotpos2 = strchr(dotpos1,dot)) != NULL) {
+		if (newlength == 1) {	/* The normal case */
+		    *dotpos2 = *minus_sign;
+		    dotpos1++;
+		} else {		/* Some multi-byte minus marker */
+		    size_t taillength = strlen(dotpos2);
+		    dotpos1 = dotpos2 + newlength;
+		    if (dotpos1 + taillength > limit)
+			int_error(NO_CARET,
+				  "format too long due to minus_sign string");
+		    /* move tail end of string out of the way */
+		    memmove(dotpos1, dotpos2 + 1, taillength);
+		    /* insert minus_sign */
+		    memcpy(dotpos2, minus_sign, newlength);
+		}
 	    }
-	    /* clear temporary variables for safety */
-	    dotpos1=NULL;
-	    dotpos2=NULL;
 	}
 
 	/* this was at the end of every single case, before: */
@@ -850,6 +1010,19 @@ gprintf(
 	++format;
     } /* for ever */
 
+done:
+
+#if (0)
+    /* Oct 2013 - Not safe because it fails to recognize LaTeX macros.	*/
+    /* For LaTeX terminals, if the user has not already provided a   	*/
+    /* format in math mode, wrap whatever we got by default in $...$ 	*/
+    if (((term->flags & TERM_IS_LATEX)) && !strchr(tempdest, '$')) {
+	*(outstring++) = '$';
+	strcat(tempdest, "$");
+	count -= 2;
+    }
+#endif
+
     /* Copy as much as fits */
     safe_strncpy(outstring, tempdest, count);
 
@@ -857,9 +1030,6 @@ gprintf(
 }
 
 /*}}} */
-#ifndef HAVE_SNPRINTF
-# undef snprintf
-#endif
 
 /* some macros for the error and warning functions below
  * may turn this into a utility function later
@@ -870,7 +1040,7 @@ do {							\
 	    current_prompt ? current_prompt : "",	\
 	    gp_input_line);				\
 } while (0)
-    
+
 #define PRINT_SPACES_UNDER_PROMPT		\
 do {						\
     const char *p;				\
@@ -1004,11 +1174,7 @@ int_error(int t_num, const char str[], va_dcl)
     va_end(args);
 #else
     fprintf(stderr, str, a1, a2, a3, a4, a5, a6, a7, a8);
-#ifdef HAVE_SNPRINTF
     snprintf(error_message, sizeof(error_message), str, a1, a2, a3, a4, a5, a6, a7, a8);
-#else
-    sprintf(error_message, str, a1, a2, a3, a4, a5, a6, a7, a8);
-#endif
 #endif
 
     fputs("\n\n", stderr);
@@ -1116,11 +1282,12 @@ graph_error(const char *fmt, va_dcl)
 /*}}} */
 
 
-/* Squash spaces in the given string (DFK) */
-/* That is, reduce all multiple white-space chars to single spaces */
-/* Done in place. Currently used only by help_command() */
+/*
+ * Reduce all multiple white-space chars to single spaces (if remain == 1)
+ * or remove altogether (if remain == 0).  Modifies the original string.
+ */
 void
-squash_spaces(char *s)
+squash_spaces(char *s, int remain)
 {
     char *r = s;	/* reading point */
     char *w = s;	/* writing point */
@@ -1129,7 +1296,7 @@ squash_spaces(char *s)
     for (w = r = s; *r != NUL; r++) {
 	if (isspace((unsigned char) *r)) {
 	    /* white space; only copy if we haven't just copied a space */
-	    if (!space) {
+	    if (!space && remain > 0) {
 		space = TRUE;
 		*w++ = ' ';
 	    }			/* else ignore multiple spaces */
@@ -1202,7 +1369,7 @@ parse_esc(char *instr)
 		    *t++ = *s++;
 		}
 	    }
-	} else if (df_separator && *s == '\"' && *(s+1) == '\"') {
+	} else if (df_separators && *s == '\"' && *(s+1) == '\"') {
 	/* EAM Mar 2003 - For parsing CSV strings with quoted quotes */
 	    *t++ = *s++; s++;
 	} else {
@@ -1245,8 +1412,20 @@ existdir (const char *name)
 #endif
 }
 
+
+TBOOLEAN
+existfile(const char *name)
+{
+#ifdef __MSC__
+    return (_access(name, 0) == 0);
+#else
+    return (access(name, F_OK) == 0);
+#endif
+}
+
+
 char *
-getusername ()
+getusername()
 {
     char *username = NULL;
 
@@ -1270,7 +1449,7 @@ TBOOLEAN contains8bit(const char *s)
 
 /* Read from second byte to end of UTF-8 sequence.
    used by utf8toulong() */
-TBOOLEAN
+static TBOOLEAN
 utf8_getmore (unsigned long * wch, const char **str, int nbytes)
 {
   int i;
@@ -1279,7 +1458,7 @@ utf8_getmore (unsigned long * wch, const char **str, int nbytes)
 
   for (i = 0; i < nbytes; i++) {
     c = (unsigned char) **str;
-  
+
     if ((c & 0xc0) != 0x80) {
       *wch = INVALID_UTF8;
       return FALSE;
@@ -1366,7 +1545,7 @@ gp_strlen(const char *s)
  * Returns a pointer to the Nth character of s
  * or a pointer to the trailing \0 if N is too large
  */
-char *
+static char *
 utf8_strchrn(const char *s, int N)
 {
     int i = 0, j = 0;
@@ -1391,24 +1570,140 @@ gp_strchrn(const char *s, int N)
 	return (char *)&s[N];
 }
 
+
 /* TRUE if strings a and b are identical save for leading or trailing whitespace */
 TBOOLEAN
 streq(const char *a, const char *b)
 {
     int enda, endb;
 
-    while (isspace(*a))
+    while (isspace((unsigned char)*a))
 	a++;
-    while (isspace(*b))
+    while (isspace((unsigned char)*b))
 	b++;
 
-    enda = strlen(a) - 1;
-    endb = strlen(b) - 1;
+    enda = (*a) ? strlen(a) - 1 : 0;
+    endb = (*b) ? strlen(b) - 1 : 0;
 
-    while (isspace(a[enda]))
+    while (isspace((unsigned char)a[enda]))
 	enda--;
-    while (isspace(b[endb]))
+    while (isspace((unsigned char)b[endb]))
 	endb--;
 
     return (enda == endb) ? !strncmp(a,b,++enda) : FALSE;
 }
+
+
+/* append string src to dest
+   re-allocates memory if necessary, (re-)determines the length of the 
+   destination string only if len==0
+ */
+size_t
+strappend(char **dest, size_t *size, size_t len, const char *src)
+{
+    size_t destlen = (len != 0) ? len : strlen(*dest);
+    size_t srclen = strlen(src);
+    if (destlen + srclen + 1 > *size) {
+	*size *= 2;
+	*dest = (char *) gp_realloc(*dest, *size, "strappend");
+    }
+    memcpy(*dest + destlen, src, srclen + 1);
+    return destlen + srclen;
+}
+
+
+/* convert a struct value to a string */
+char *
+value_to_str(struct value *val, TBOOLEAN need_quotes)
+{
+    static int i = 0;
+    static char * s[4] = {NULL, NULL, NULL, NULL};
+    static size_t c[4] = {0, 0, 0, 0};
+    static const int minbufsize = 54;
+    int j = i;
+
+    i = (i + 1) % 4;
+    if (s[j] == NULL) {
+	s[j] = (char *) gp_alloc(minbufsize, "value_to_str");
+	c[j] = minbufsize;
+    }
+
+    switch (val->type) {
+    case INTGR:
+	sprintf(s[j], "%d", val->v.int_val);
+	break;
+    case CMPLX:
+	if (isnan(val->v.cmplx_val.real))
+	    sprintf(s[j], "NaN");
+	else if (val->v.cmplx_val.imag != 0.0)
+	    sprintf(s[j], "{%s, %s}",
+	            num_to_str(val->v.cmplx_val.real),
+	            num_to_str(val->v.cmplx_val.imag));
+	else
+	    return num_to_str(val->v.cmplx_val.real);
+	break;
+    case STRING:
+	if (val->v.string_val) {
+	    if (!need_quotes) {
+		return val->v.string_val;
+	    } else {
+		char * cstr = conv_text(val->v.string_val);
+		size_t reqsize = strlen(cstr) + 3;
+		if (reqsize > c[j]) {
+		    /* Don't leave c[j[ non-zero if realloc fails */
+		    s[j] = (char *) gp_realloc(s[j], reqsize + 20, NULL);
+		    if (s[j] != NULL) {
+			c[j] = reqsize + 20;
+		    } else {
+			c[j] = 0;
+			int_error(NO_CARET, "out of memory");
+		    }
+		}
+		sprintf(s[j], "\"%s\"", cstr);
+	    }
+	} else {
+	    s[j][0] = NUL;
+	}
+	break;
+    case DATABLOCK:
+	{
+	char **dataline = val->v.data_array;
+	int nlines = 0;
+	if (dataline != NULL) {
+	    while (*dataline++ != NULL)
+		nlines++;
+	}
+	sprintf(s[j], "<%d line data block>", nlines);
+	break;
+	}
+    default:
+	int_error(NO_CARET, "unknown type in value_to_str()");
+    }
+
+    return s[j];
+}
+
+
+/* Helper for value_to_str(): convert a single number to decimal
+ * format. Rotates through 4 buffers 's[j]', and returns pointers to
+ * them, to avoid execution ordering problems if this function is
+ * called more than once between sequence points. */
+char *
+num_to_str(double r)
+{
+    static int i = 0;
+    static char s[4][25];
+    int j = i++;
+
+    if (i > 3)
+	i = 0;
+
+    sprintf(s[j], "%.15g", r);
+    if (strchr(s[j], '.') == NULL &&
+	strchr(s[j], 'e') == NULL &&
+	strchr(s[j], 'E') == NULL)
+	strcat(s[j], ".0");
+
+    return s[j];
+}
+
